
internal void
APP_updateAndRender( APP_STATE * appState, PLATFORM * platform ) {
	BACKBUFFER * backbuffer = &platform->backbuffer;
	VIEWPORT   * camera     = &appState->camera;
	
	SAVE_STATE * saveState = &appState->saveState;
	
	KEYBOARD_STATE * keyboard = &platform->keyboard;
	
	MEMORY * tempMemory = &platform->tempMemory;
	
	vec2 app_dim     = appState->app_dim;
	vec2 app_halfDim = appState->app_halfDim;
	rect app_bound   = appState->app_bound;
	
	PLAYER_STATE * player = &appState->player;
	SPLATTER_BUFFER * splatterBuffer = &appState->splatterBuffer;
	
	if( !appState->isInitialized ) {
		appState->isInitialized = true;
		
		player->halfDim = 10.0f;
		camera->screenCenter = app_halfDim;
		
		FILE_DATA fontFile = platform->readFile( &platform->permMemory, "fonts.pak" );
		Assert( fontFile.contents );
		
		uint8 * ptr = ( uint8 * )fontFile.contents;
		appState->font_xlg = ( FONT * )ptr;   ptr += sizeof( FONT );   alignFont( appState->font_xlg, &ptr );
		appState->font_lg  = ( FONT * )ptr;   ptr += sizeof( FONT );   alignFont( appState->font_lg,  &ptr );
		appState->font_med = ( FONT * )ptr;   ptr += sizeof( FONT );   alignFont( appState->font_med, &ptr );
		appState->font_sm  = ( FONT * )ptr;   ptr += sizeof( FONT );   alignFont( appState->font_sm,  &ptr );
		
		FILE_DATA saveFile = platform->readFile( tempMemory, "save.dat" );
		if( saveFile.contents ) {
			memcpy( saveState, saveFile.contents, sizeof( SAVE_STATE ) );
			_popSize( tempMemory, saveFile.sizeInBytes );
		} else {
			memset( saveState, 0, sizeof( SAVE_STATE ) );
		
			saveState->tutorial_startMenu_isActive = true;
			saveState->tutorial_replay_isActive    = true;
			saveState->tutorial_levelOne_isActive  = true;
			saveState->tutorial_levelTwo_isActive  = true;
		}
		
		setAppMode( appState, appMode_startMenu );
	}
	
	FONT * font_sm  = appState->font_sm;
	FONT * font_med = appState->font_med;
	FONT * font_lg  = appState->font_lg;
	FONT * font_xlg = appState->font_xlg;
	
	boo32 left_wasPressed  = wasPressed( keyboard, keyCode_left  );
	boo32 right_wasPressed = wasPressed( keyboard, keyCode_right );
	boo32 up_wasPressed    = wasPressed( keyboard, keyCode_up    );
	boo32 down_wasPressed  = wasPressed( keyboard, keyCode_down  );
	
	boo32 left_wasReleased   = wasReleased( keyboard, keyCode_left  );
	boo32 right_wasReleased  = wasReleased( keyboard, keyCode_right );
	boo32 select_wasReleased = wasReleased( keyboard, keyCode_space );
	boo32 back_wasReleased   = wasReleased( keyboard, keyCode_escape );
	
	boo32 jump_wasPressed  = wasPressed( keyboard, keyCode_space );
	boo32 fast_wasPressed  = wasPressed( keyboard, keyCode_shift );
	
	boo32 jump_wasReleased   = wasReleased( keyboard, keyCode_space );
	boo32 fast_wasReleased   = wasReleased( keyboard, keyCode_shift );
	boo32 replay_wasReleased = wasReleased( keyboard, keyCode_w );
	
	if( fast_wasPressed   ) { appState->fast_isDown  = true;  }
	if( fast_wasReleased  ) { appState->fast_isDown  = false; }
	if( jump_wasPressed   ) { appState->jump_isDown  = true;  }
	if( jump_wasReleased  ) { appState->jump_isDown  = false; }
	if( left_wasPressed   ) { appState->left_isDown  = true;  }
	if( left_wasReleased  ) { appState->left_isDown  = false; }
	if( right_wasPressed  ) { appState->right_isDown = true;  }
	if( right_wasReleased ) { appState->right_isDown = false; }
	
	boo32 jump_isDown  = appState->jump_isDown;
	boo32 fast_isDown  = appState->fast_isDown;
	boo32 left_isDown  = appState->left_isDown;
	boo32 right_isDown = appState->right_isDown;
	
	switch( appState->appMode ) {
		case appMode_startMenu: {
			if( !appState->appMode_isInitialized ) {
				appState->appMode_isInitialized = true;
				
				appState->button_select = 0;
				appState->fadeIn  = true;
				
				appState->tutorial_toggleState = true;
				appState->tutorial_timer       = 0.0f;
				
				appState->startMenu_clearTimes = false;
			}
			clearBackbuffer( backbuffer, COLOR_WHITE );
			
			int32 button_count = 3;
			if( left_wasReleased ) {
				appState->button_select = ( appState->button_select + button_count - 1 ) % button_count;
			}
			if( right_wasReleased ) {
				appState->button_select = ( appState->button_select + button_count + 1 ) % button_count;
			}
			if( ( left_wasReleased || right_wasReleased ) && ( appState->button_select == 0 ) ) {
				appState->tutorial_toggleState = true;
				appState->tutorial_timer       = 0.0f;
			}
			
			vec2 title_position = Vec2( 620.0f, 600.0f );
			drawStringCentered( backbuffer, font_xlg, "BLOOD", title_position );
			drawStringCentered( backbuffer, font_xlg, "BOY",   title_position + Vec2( 0.0f, -240.0f ) );
			
			vec2 startGame_position = Vec2( 620.0f, 120.0f );
			rect startGame_bound    = rectCD( startGame_position, Vec2( 400.0f, 100.0f ) );
			drawStringCentered( backbuffer, font_lg, "Start Game", startGame_position );
			if( appState->button_select == 0 ) {
				drawRectOutline( backbuffer, startGame_bound, COLOR_RED );
				if( select_wasReleased ) {
					appState->fadeOut = true;
				}
			}
			
			rect exit_bound = rectBLD( Vec2( 1070.0f, 40.0f ), Vec2( 150.0f, 100.0f ) );
			drawStringCentered( backbuffer, font_lg, "Exit", getCenter( exit_bound ) );
			if( appState->button_select == 1 ) {
				drawRectOutline( backbuffer, exit_bound, COLOR_RED );
				if( select_wasReleased ) {
					appState->appMode = appMode_exitGame;
				}
			}
			
			rect eraseSave_bound = rectBLD( Vec2( 20.0f, 40.0f ), Vec2( 240.0f, 80.0f ) );
			if( appState->startMenu_clearTimes ) {
				appState->startMenu_timer += appState->targetSec;
				if( appState->startMenu_timer >= 0.35f ) {
					appState->startMenu_clearTimes = false;
				}
			}
			if( appState->button_select == 2 ) {
				drawRectOutline( backbuffer, eraseSave_bound, COLOR_RED );
				if( select_wasReleased ) {
					memset( saveState->level_bestTime, 0, sizeof( flo32 ) * 20 );
					saveGame( platform, saveState );
					appState->startMenu_clearTimes = true;
					appState->startMenu_timer = 0.0f;
					appState->level_select = 0;
				}
				if( appState->startMenu_clearTimes ) {
					flo32 ratio = minValue( appState->startMenu_timer / 0.25f, 1.0f );
					rect  bound = rectBLD( getBL( eraseSave_bound ), Vec2( 240.0f * ratio, 80.0f ) );
					drawRect( backbuffer, bound, COLOR_RED );
				}
			}
			drawStringCentered( backbuffer, font_med, "Clear Times", getCenter( eraseSave_bound ) );

			if( saveState->tutorial_startMenu_isActive ) {
				if( ( appState->tutorial_toggleState ) && ( appState->button_select == 0 ) ) {
					drawStringCentered( backbuffer, font_sm, "Press SPACEBAR", getTC( startGame_bound ) + Vec2( 0.0f, 16.0f ) );
				}
				
				appState->tutorial_timer += appState->targetSec;
				if( appState->tutorial_timer >= 0.8f ) {
					appState->tutorial_toggleState = !appState->tutorial_toggleState;
					appState->tutorial_timer = 0.0f;
				}
			}
			
			updateFadeIn(  backbuffer, appState );
			updateFadeOut( backbuffer, appState );
			if( appState->exitMode ) {
				appState->exitMode = false;
				
				saveState->tutorial_startMenu_isActive = false;
				setAppMode( appState, appMode_levelMenu );
			}
		} break;
		
		case appMode_mainGame: {
			clearBackbuffer( backbuffer, COLOR_WHITE );
			if( !player->isDead ) {
				boo32 player_isOnGround       = false;
				boo32 player_isOnWall_onRight = false;
				boo32 player_isOnWall_onLeft  = false;
				
				vec2 playerPos = player->position;
				
				GROUND_STATE * groundState = &appState->groundState;
				CRUMBLE_STATE * crumbleState = &appState->crumbleState;
				
				testMovement( &player_isOnGround, &player_isOnWall_onLeft, &player_isOnWall_onRight,  groundState->bound,  groundState->count, playerPos, player->halfDim );
				testMovement( &player_isOnGround, &player_isOnWall_onLeft, &player_isOnWall_onRight, crumbleState->bound, crumbleState->count, playerPos, player->halfDim );
				testTimer( &appState->crumbleState );
				testContact( crumbleState, player->position, player->halfDim );
			
				boo32 attemptJump = jump_wasPressed;
				flo32 horizontalAccel = ( fast_isDown ? 2000.0f : 1000.0f );
				vec2 friction = { 2.5, 0.0f };
				if( player_isOnGround && !attemptJump ) {
					horizontalAccel *= 4.0f;
					friction.x *= 4.0f;
				}
				if( player->velocity_perSecond.y < 0.0f ) {
					friction.y = 2.5f;
					if( player_isOnWall_onLeft || player_isOnWall_onRight ) {
						friction.y = 10.0f;
					}
				}
				vec2 playerAccel = {};
				if( right_isDown ) {
					if( !player_isOnGround && player_isOnWall_onLeft && player->stickyTime > 0.0f ) {
						player->stickyTime -= appState->targetSec;
					} else {
						player->stickyTime = 0.15f;
						playerAccel.x += horizontalAccel;
					}
				}
				if( left_isDown ) {
					if( !player_isOnGround && player_isOnWall_onRight && player->stickyTime > 0.0f ) {
						player->stickyTime -= appState->targetSec;
					} else {
						player->stickyTime = 0.15f;
						playerAccel.x -= horizontalAccel;
					}
				}
				if( attemptJump ) {
					attemptJump = false;
					if( player_isOnGround || player_isOnWall_onRight || player_isOnWall_onLeft ) {
						player->velocity_perSecond.y = 400.0f;
						player->airTime = 0.25f;
						if( !player_isOnGround ) {
							flo32 velocityX_wallJump = ( fast_isDown ? 600.0f : 300.0f );
							if( player_isOnWall_onRight ) {
								player->velocity_perSecond.x = -velocityX_wallJump;
							} else if( player_isOnWall_onLeft ) {
								player->velocity_perSecond.x = velocityX_wallJump;
							}
						}
					}
				}
				if( jump_wasReleased ) {
					if( player->airTime > 0.0f ) {
						player->velocity_perSecond.y = 200.0f;
					}
					player->airTime = 0.0f;
				}
				if( jump_isDown && player->airTime > 0.0f ) {
					flo32 airTimeIncrement = appState->targetSec;
					if( player_isOnWall_onLeft || player_isOnWall_onRight ) {
						airTimeIncrement *= 0.75f;
					}
					player->airTime -= airTimeIncrement;
				} else {
					playerAccel.y = -2000.0f;
				}
				
				playerAccel.x -= player->velocity_perSecond.x * friction.x;
				playerAccel.y -= player->velocity_perSecond.y * friction.y;
				
				player->velocity_perSecond += playerAccel * appState->targetSec;
			}
			updateActiveEntities( appState );
			updateSplatter( splatterBuffer, appState->targetSec );
			
			if( !player->isDead ) {
				flo32 totalTMin = 1.0f;
				flo32 tMinUsed = 0.0f;
				for( uint32 iteration = 0; ( ( iteration < 4 ) || ( tMinUsed < 1.0f ) ); iteration++ ) {
					vec2 collisionNormal = {};
					flo32 tMin_forThisIteration = totalTMin - tMinUsed;
					
					flo32 time = appState->targetSec * tMin_forThisIteration;
					vec2 newPlayerPosition = player->velocity_perSecond * time + player->position;
					vec2 playerMoveVector = newPlayerPosition - player->position;
					
					GROUND_STATE * groundState = &appState->groundState;
					for( int32 ground_index = 0; ground_index < groundState->count; ground_index++ ) {
						rect bound    = groundState->bound[ ground_index ];
						rect adjBound = addRadius( bound, player->halfDim );
						testEntityCollision( player->position, playerMoveVector, adjBound, &tMin_forThisIteration, &collisionNormal );
					}
					
					CRUMBLE_STATE * crumbleState = &appState->crumbleState;
					for( int32 crumble_index = 0; crumble_index < crumbleState->count; crumble_index++ ) {
						rect bound    = crumbleState->bound[ crumble_index ];
						rect adjBound = addRadius( bound, player->halfDim );
						testEntityCollision( player->position, playerMoveVector, adjBound, &tMin_forThisIteration, &collisionNormal );
					}
					
					player->position += playerMoveVector * tMin_forThisIteration;
					player->velocity_perSecond -= collisionNormal * dot( player->velocity_perSecond, collisionNormal );
					tMinUsed += tMin_forThisIteration;
				}
			}
			
			SAWBLADE_STATE * sawbladeState = &appState->sawbladeState;
			for( int32 sawblade_index = 0; ( !player->isDead ) && ( sawblade_index < sawbladeState->count ); sawblade_index++ ) {
				SAWBLADE sawblade = sawbladeState->sawblade[ sawblade_index ];
				if( rectCircleCollision( rectCR( player->position, player->halfDim ), sawblade.position, sawblade.radius ) ) {
					killPlayer( appState );
				}
			}
			
			KILL_STATE * killState = &appState->killState;
			for( int32 kill_index = 0; ( !player->isDead ) && ( kill_index < killState->count ); kill_index++ ) {
				rect    bound = killState->bound[ kill_index ];
				rect adjBound = addRadius( bound, player->halfDim );
				if( isInBound( player->position, adjBound ) ) {
					killPlayer( appState );
				}
			}
			
			camera->position = clipCameraPosition( global_level_APP_STATE[ appState->currentActiveLevel ].cameraBound, player->position );
			
			if( !player->isDead ) {
				rect player_bound = addDim( rectCD( camera->position, app_dim ), 80.0f, 30.0f, 80.0f, 120.0f );
				if( !isInBound( player->position, player_bound ) ) {
					killPlayer( appState );
				}
			}
			boo32 levelIsComplete = false;
			if( !player->isDead ) {
				appState->replay_data[ appState->replay_currentFrameIndex ] = player->position;
				appState->replay_currentFrameIndex++;
				if( appState->replay_currentFrameIndex >= MAX_REPLAY_DATA ) {
					appState->replay_currentFrameIndex = 0;
					appState->replay_attemptCount = 0;
				}
				
				rect adjGoalBound = addRadius( global_level_APP_STATE[ appState->currentActiveLevel ].goalBound, player->halfDim );
				if( isInBound( player->position, adjGoalBound ) ) {
					levelIsComplete = true;
					
					if( appState->currentActiveLevel == 0 ) {
						saveState->tutorial_levelOne_isActive = false;
					}
					if( appState->currentActiveLevel == 1 ) {
						saveState->tutorial_levelTwo_isActive = false;
					}
					
					flo32 currentBestTime = saveState->level_bestTime[ appState->currentActiveLevel ];
					if( ( currentBestTime == 0.0f ) || ( currentBestTime > appState->level_currentTime ) ) {
						currentBestTime = appState->level_currentTime;
					}
					saveState->level_bestTime[ appState->currentActiveLevel ] = currentBestTime;
					
					appState->tutorial_timer = 0.0f;
					
					saveGame( platform, saveState );
					
					endReplayAttempt( appState );
					initReplay( appState );
				}
			}
			if( !levelIsComplete ) {
				appState->level_currentTime += appState->targetSec;
			}
			
			if( !player->isDead ) {
				drawBox( backbuffer, camera, rectCR( player->position, player->halfDim ), COLOR_BLACK, COLOR_RED );
			}
			drawActiveEntities( backbuffer, appState );
			drawSplatter( backbuffer, camera, splatterBuffer );
			drawTimeText( backbuffer, appState );
			
			tutorial_levelOne( backbuffer, appState );
			tutorial_levelTwo( backbuffer, appState );
			
			if( player->isDead ) {
				if( appState->level_reloadTimer >= 0.75f ) {
					appState->fadeOut = true;
				}
				appState->level_reloadTimer += appState->targetSec;
			}
			updateFadeIn(  backbuffer, appState );
			updateFadeOut( backbuffer, appState );
			if( appState->exitMode ) {
				appState->exitMode = false;
				resetLevel( appState );
			}
			if( back_wasReleased ) {
				appState->appMode = appMode_exitLevel;
			}
		} break;
		
		case appMode_levelReplay: {
			clearBackbuffer( backbuffer, COLOR_WHITE );
			
			updateSplatter( splatterBuffer, appState->targetSec );
			updateActiveEntities( appState );
			
			uint32 cameraLookupIndex = appState->replay_attemptData[ appState->replay_attemptCount - 1 ].startIndex + appState->replay_currentFrameIndex;
			camera->position = clipCameraPosition( global_level_APP_STATE[ appState->currentActiveLevel ].cameraBound, appState->replay_data[ cameraLookupIndex ] );
			
			drawActiveEntities( backbuffer, appState );
			drawSplatter( backbuffer, camera, splatterBuffer );
			
			flo32 backbufferHeight = ( flo32 )backbuffer->height;
			vec2 atPos = { 10.0f, backbufferHeight - 10.0f - font_med->ascent };
			drawString( backbuffer, font_med, "W : Replay Level", atPos );
			
			atPos.y -= font_med->advanceHeight;
			drawString( backbuffer, font_med, "Esc : Exit to Menu", atPos );
			
			if( !tutorial_replay( backbuffer, appState ) ) {
				drawStringCentered( backbuffer, font_lg, "SPACEBAR : Next Level", Vec2( app_halfDim.x, 100.0f ) );
			}
			drawTimeText( backbuffer, appState );
			
			for( uint32 attemptIndex = 0; attemptIndex < appState->replay_attemptCount; attemptIndex++ ) {
				ATTEMPT_DATA attempt = appState->replay_attemptData[ attemptIndex ];
				if( appState->replay_currentFrameIndex < attempt.frameCount ) {
					uint32 currentLookupIndex = appState->replay_attemptData[ attemptIndex ].startIndex + appState->replay_currentFrameIndex;
					vec2 position = appState->replay_data[ currentLookupIndex ];
					drawBox( backbuffer, &appState->camera, rectCR( position, player->halfDim ), COLOR_BLACK, COLOR_RED );
				}
				if( appState->replay_currentFrameIndex == attempt.frameCount ) {
					pushNewSplatter( splatterBuffer, attempt.endPosition, appState->targetSec );
				}
			}
			
			appState->replay_currentFrameIndex++;
			if( appState->replay_currentFrameIndex == appState->replay_attemptData[ appState->replay_attemptCount - 1 ].frameCount ) {
				resetReplay( appState );
			}
			
			if( back_wasReleased )   { appState->button_select = 0;  appState->fadeOut = true; }
			if( replay_wasReleased ) { appState->button_select = 1;  appState->fadeOut = true; }
			if( jump_wasReleased )   { appState->button_select = 2;  appState->fadeOut = true; }
			
			updateFadeOut( backbuffer, appState );
			if( appState->exitMode ) {
				appState->exitMode = false;
				saveState->tutorial_replay_isActive = false;
				switch( appState->button_select ) {
					case 0: {
						setAppMode( appState, appMode_levelMenu );
						appState->level_select = appState->currentActiveLevel;
					} break;
					
					case 1: {
						initLevel( appState, appState->currentActiveLevel );
					} break;
					
					case 2: {
						if( appState->currentActiveLevel == 19 ) {
							setAppMode( appState, appMode_levelMenu );
							appState->level_select = appState->currentActiveLevel;
						} else {
							initLevel( appState, appState->currentActiveLevel + 1 );
						}
					} break;
				}
				appState->button_select = 0;
			}
		} break;
		
		case appMode_exitLevel:
		case appMode_exitGame: {
			Assert( ( appState->button_select == 0 ) || ( appState->button_select == 1 ) );
			if( left_wasReleased || right_wasReleased ) {
				appState->button_select = ( appState->button_select + 1 ) % 2;
			}
			
			rect menu_bound = rectCD( app_halfDim, Vec2( 440.0f, 400.0f ) );
			drawRect(        backbuffer, menu_bound, COLOR_WHITE );
			drawRectOutline( backbuffer, menu_bound, COLOR_BLACK );
			
			vec2 label_position = getTC( menu_bound ) + Vec2( 0.0f, -100.0f );
			if( appState->appMode == appMode_exitGame ) {
				drawStringCentered( backbuffer, font_lg, "Exit Game?",  label_position );
			} else {
				drawStringCentered( backbuffer, font_lg, "Exit Level?", label_position );
			}
			vec2 button_dim = Vec2( 180.0f, 100.0f );
			
			rect yesButton_bound = rectBLD( getBL( menu_bound ) + Vec2( 20.0f, 20.0f ), button_dim );
			drawStringCentered( backbuffer, font_lg, "Yes", getCenter( yesButton_bound ) );
			
			rect noButton_bound = rectBRD( getBR( menu_bound ) + Vec2( -20.0f, 20.0f ), button_dim );
			drawStringCentered( backbuffer, font_lg, "No", getCenter( noButton_bound ) );
			
			rect hotButton_bound = ( appState->button_select == 0 ) ? yesButton_bound : noButton_bound;
			drawRectOutline( backbuffer, hotButton_bound, COLOR_RED );
			
			updateFadeOut( backbuffer, appState );
			if( select_wasReleased ) {
				// 0 : YES, 1 : NO
				if( appState->button_select == 0 ) {
					appState->fadeOut = true;
				} else {
					if( appState->appMode == appMode_exitGame ) {
						appState->appMode = appMode_startMenu;
					} else {
						appState->appMode = appMode_mainGame;
					}
				}
			}
			if( appState->exitMode ) {
				appState->exitMode = false;
				if( appState->appMode == appMode_exitGame ) {
					appState->isRunning = false;
				} else {
					appState->level_select = appState->currentActiveLevel;
					setAppMode( appState, appMode_levelMenu );
				}
			}
		} break;
		
		case appMode_levelMenu: {
			vec2  level_dim    = Vec2( 100.0f, 80.0f );
			flo32 level_offset = 20.0f;
			
			vec2 level_pos[ 20 ] = {};
			
			int32 level_count = 0;
			vec2 atPos = app_halfDim + Vec2( -( ( 3.0f * level_dim.x ) + ( 2.5f * level_offset ) ), 120.0f );
			for( int32 iter = 0; iter < 5; iter++ ) { level_pos[ level_count ] = atPos;   atPos.x += ( level_dim.x + level_offset );   level_count++; }
			for( int32 iter = 0; iter < 2; iter++ ) { level_pos[ level_count ] = atPos;   atPos.y -= ( level_dim.y + level_offset );   level_count++; }
			for( int32 iter = 0; iter < 5; iter++ ) { level_pos[ level_count ] = atPos;   atPos.x -= ( level_dim.x + level_offset );   level_count++; }
			for( int32 iter = 0; iter < 2; iter++ ) { level_pos[ level_count ] = atPos;   atPos.y -= ( level_dim.y + level_offset );   level_count++; }
			for( int32 iter = 0; iter < 6; iter++ ) { level_pos[ level_count ] = atPos;   atPos.x += ( level_dim.x + level_offset );   level_count++; }
			
			if( !appState->appMode_isInitialized ) {
				appState->appMode_isInitialized = true;
				appState->fadeIn = true;
				
				appState->token_timer = 0.0f;
				
				appState->token_position = level_pos[ appState->level_select ];
			}
			clearBackbuffer( backbuffer, COLOR_WHITE );
			
			if( !appState->token_isMoving ) {
				int32 select = appState->level_select;
				boo32 move_nextLevel = ( left_wasPressed  && (   select >= 7 && select <= 11                                       ) ) || 
				                       ( right_wasPressed && ( ( select >= 0 && select <=  4 ) || ( select >= 14 && select <= 18 ) ) ) ||
				                       ( down_wasPressed  && ( ( select >= 5 && select <=  6 ) || ( select >= 12 && select <= 13 ) ) );
				boo32 move_prevLevel = ( left_wasPressed  && ( ( select >= 1 && select <=  5 ) || ( select >= 15 && select <= 19 ) ) ) ||
				                       ( right_wasPressed && (   select >= 8 && select <= 12                                       ) ) ||
					                    ( up_wasPressed    && ( ( select >= 6 && select <=  7 ) || ( select >= 13 && select <= 14 ) ) );
				
				if( ( move_nextLevel ) || ( move_prevLevel ) ) {
					appState->level_select = ( move_nextLevel ) ? ( appState->level_select + 1 ) : ( appState->level_select - 1 );
					appState->token_isMoving = true;
					appState->token_moveVector = level_pos[ appState->level_select ] - appState->token_position;
				}
				if( select_wasReleased ) {
					appState->fadeOut = true;
					appState->tutorial_toggleState = true;
				}
			} else {
				appState->token_timer += appState->targetSec;
				if( appState->token_timer >= 0.15f ) {
					appState->token_isMoving = false;
					appState->token_position = level_pos[ appState->level_select ];
					appState->token_timer = 0.0f;
				}
			}
			if( back_wasReleased ) {
				appState->fadeOut = true;
				appState->tutorial_toggleState = false;
			}
			
			char string_buffer[ 128 ] = {};
			for( int32 level_index = 0; level_index < 20; level_index++ ) {
				vec2 pos   = level_pos[ level_index ];
				rect bound = rectCD( pos, level_dim );
				
				uint32 fill_color = COLOR_WHITE;
				
				flo32 bestTime   = saveState->level_bestTime[ level_index ];
				flo32 targetTime = global_level_APP_STATE[ level_index ].targetTime;
				
				if( bestTime > targetTime ) {
					fill_color = COLOR_LEVEL_COMPLETE;
				} else if( bestTime != 0.0f ) {
					fill_color = COLOR_TARGET_TIME_COMPLETE;
				}
				
				drawBox( backbuffer, bound, COLOR_BLACK, fill_color );
				
				sprintf( string_buffer, "%d", level_index + 1 );
				drawStringCentered( backbuffer, font_lg, string_buffer, pos );
				
				if( ( bestTime != 0.0f ) && ( bestTime < targetTime ) ) {
					drawLine( backbuffer, getTL( bound ), getBR( bound ), COLOR_BLACK );
					drawLine( backbuffer, getTR( bound ), getBL( bound ), COLOR_BLACK );
				}
			}
			
			flo32 token_ratio    = minValue( appState->token_timer / 0.15f, 1.0f );
			vec2  token_position = appState->token_position + appState->token_moveVector * token_ratio;
			rect  token_bound    = rectCD( token_position + Vec2( 0.0f, 14.0f ), Vec2( 80.0f, 80.0f ) );
			drawBox( backbuffer, token_bound, COLOR_BLACK, COLOR_RED );
			
			vec2 basePos = Vec2( app_halfDim.x - 120.0f, app_dim.y - ( 20.0f + font_med->ascent ) );
			flo32 bestTime = saveState->level_bestTime[ appState->level_select ];
			if( bestTime > 0.0f ) {
				sprintf( string_buffer, "Best Time: %.02f", bestTime );
			} else {
				sprintf( string_buffer, "Best Time: --.--" );
			}
			drawString( backbuffer, font_med, string_buffer, basePos );
			
			sprintf( string_buffer, "Target Time: %.02f", global_level_APP_STATE[ appState->level_select ].targetTime );
			drawString( backbuffer, font_med, string_buffer, basePos + Vec2( 0.0f, -font_med->advanceHeight ) );
			
			flo32 totalTime = 0.0f;
			for( int32 level_index = 0; level_index < 20; level_index++ ) {
				totalTime += saveState->level_bestTime[ level_index ];
			}
			sprintf( string_buffer, "Total Time: %0.2f", totalTime );
			drawString( backbuffer, font_med, string_buffer, basePos + Vec2( 400.0f, 0.0f ) );
			
			updateFadeIn(  backbuffer, appState );
			updateFadeOut( backbuffer, appState );
			if( appState->exitMode ) {
				appState->exitMode = false;
				if( appState->tutorial_toggleState ) {
					initLevel( appState, appState->level_select );
				} else {
					setAppMode( appState, appMode_startMenu );
				}
			}
		} break;
	}
}