
//----------
// TODO
//  - level editor
//  - level packer

#define APP_targetFPS	60.0f
#define APP_permMemorySize		_MB( 3 )
#define APP_tempMemorySize		_MB( 1 )

#define COLOR_BLACK        ( 0xFF000000 )
#define COLOR_WHITE        ( 0xFFFFFFFF )
#define COLOR_TRANSPARENT  ( 0x00000000 )
#define COLOR_RED          ( 0xFFFF0000 )
#define COLOR_GREEN        ( 0xFF00FF00 )
#define COLOR_BLUE         ( 0xFF0000FF )
#define COLOR_PINK         ( 0xFFFFA0A0 )
#define COLOR_KILL         ( 0xFFFFAF78 )
#define COLOR_YELLOW       ( 0xFFFFFF00 )
#define COLOR_GOAL         ( 0xFFFFA0A0 )
#define COLOR_LEVEL_COMPLETE        ( 0xFFF01414 )
#define COLOR_TARGET_TIME_COMPLETE  ( 0xFFAA0000 )

#define __reinterpretToUInt( var ) ( uint32 )( *( ( flo32 * )&var ) );

struct FILE_DATA {
	void * contents;
	uint32 sizeInBytes;
};

typedef FILE_DATA ( READ_FILE  )( MEMORY * memory, char * filename );
typedef boo32     ( WRITE_FILE )( char * filename, void * data, uint32 size ); 

struct PLATFORM {
	KEYBOARD_STATE keyboard;
	
	READ_FILE  * readFile;
	WRITE_FILE * writeFile;
	
	MEMORY permMemory;
	MEMORY tempMemory;
	flo32 targetSec;
	
	BACKBUFFER backbuffer;
};

enum levelData_ENTITY_ID {
	entityID_none,
	
	entityID_ground,
	entityID_crumble,
	entityID_kill,
	
	entityID_sawBlade,
	entityID_movingSawBlade,
	entityID_rotatingSawBlade,
	entityID_flyingSawBlade,
	
	entityID_count,
};

struct entity_groundData {
	uint32 entityID;
	rect terrainBound;
};

struct entity_crumble_readData {
	uint32 entityID;
	rect terrainBound;
};

struct entity_kill_readData {
	uint32 entityID;
	rect terrainBound;
};

struct entity_sawBladeData {
	uint32 entityID;
	vec2 position;
	flo32 radius;
};

struct entity_movingSawBlade_readData {
	uint32 entityID;
	flo32 radius;
	vec2 position;
	vec2 moveVector_perSecond;
	flo32 targetHalfCycleTime;
};

struct entity_rotatingSawBlade_readData {
	uint32 entityID;
	flo32 bladeRadius;
	flo32 armRadius;
	vec2 centerPosition;
	flo32 radiansPerSecond;
};

struct entity_flyingSawBlade_readData {
	uint32 entityID;
	flo32 radius;
	uint32 sawBladeCount;
	vec2 startPosition;
	vec2 moveVector_perSecond;
	flo32 targetFlightTime;
	flo32 targetLaunchTime;
};

enum APP_MODE {
	appMode_null,
	
	appMode_startMenu,
	appMode_levelMenu,
	appMode_mainGame,
	appMode_levelReplay,
	appMode_exitGame,
	appMode_exitLevel,
	
	appMode_count,
};

struct PLAYER_STATE {
	vec2 position;
	vec2 velocity_perSecond;
	flo32 halfDim;
	flo32 airTime;
	flo32 stickyTime;
	boo32 isDead;
};

struct ATTEMPT_DATA {
	uint32 startIndex;
	uint32 frameCount;
	vec2 endPosition;
};

struct LEVEL_DATA {
	uint32 entity_count;
	uint32 entity_baseIndex;
	vec2 spawnPosition;
	rect cameraBound;
	rect goalBound;
	flo32 targetTime;
	flo32 bestTime;
};

enum TUTORIAL_STATE {
	tutorial_isOff,
	
	tutorial_startMenu_enter,
	
	tutorial_levelOne_move,
	tutorial_levelOne_jump,
	tutorial_levelOne_goal,
	
	tutorial_levelTwo_moveFast,
};

struct SAVE_STATE {
	flo32 level_bestTime[ 20 ];
	
	boo32 tutorial_startMenu_isActive;
	boo32 tutorial_levelOne_isActive;
	boo32 tutorial_levelTwo_isActive;
	boo32 tutorial_replay_isActive;
};

struct APP_STATE {
	boo32 isRunning;
	boo32 isInitialized;
	
	vec2 app_dim;
	vec2 app_halfDim;
	rect app_bound;
	
	FONT * font_xlg;
	FONT * font_lg;
	FONT * font_med;
	FONT * font_sm;
	
	VIEWPORT camera;
	
	APP_MODE appMode;
	boo32    appMode_isInitialized;
	
	int32 button_select;
	int32 level_select;
	
	flo32 targetSec;
	int32 currentActiveLevel;
	
	PLAYER_STATE player;
	boo32 jump_isDown;
	boo32 fast_isDown;
	boo32 left_isDown;
	boo32 right_isDown;
	SPLATTER_BUFFER splatterBuffer;
	flo32 level_currentTime;
	flo32 level_reloadTimer;
	
	flo32 levelSelect_totalTime;
	
	boo32 token_isMoving;
	flo32 token_timer;
	vec2  token_position;
	vec2  token_moveVector;
	
	GROUND_STATE     groundState;
	CRUMBLE_STATE   crumbleState;
	SAWBLADE_STATE sawbladeState;
	KILL_STATE         killState;

#define MAX_ATTEMPT 8192
#define MAX_REPLAY_DATA 432000 // 2 hours
	ATTEMPT_DATA replay_attemptData[ MAX_ATTEMPT ];
	uint32 replay_attemptCount;
	uint32 replay_currentFrameIndex;
	vec2 replay_data[ MAX_REPLAY_DATA ];
	
	boo32 fadeIn;
	flo32 fadeIn_timer;
	boo32 fadeOut;
	flo32 fadeOut_timer;
	boo32 exitMode;
	
	flo32 tutorial_timer;
	boo32 tutorial_toggleState;
	int32 tutorial_mode;
	
	boo32 startMenu_clearTimes;
	flo32 startMenu_timer;
	
	SAVE_STATE saveState;
};