
#define CRUMBLE__MAX_COUNT        ( 64 )
#define CRUMBLE__MAX_TIMER_COUNT  ( 32 )
struct CRUMBLE_STATE {
	int32 timer_count;
	flo32 timer[ CRUMBLE__MAX_TIMER_COUNT ];
	
	int32 count;
	rect  bound[ CRUMBLE__MAX_COUNT ];
};

internal void
updateTimer( CRUMBLE_STATE * crumbleState, flo32 dt ) {
	for( int32 crumble_index = 0; crumble_index < crumbleState->timer_count; crumble_index++ ) {
		crumbleState->timer[ crumble_index ] += dt;
	}
}

internal void
testContact( CRUMBLE_STATE * crumbleState, vec2 player_position, flo32 player_radius ) {
	for( int32 crumble_index = 0; crumble_index < crumbleState->count; ) {
		rect bound = crumbleState->bound[ crumble_index ];
		rect adjBound = addRadius( bound, player_radius );
		if( isInBound( player_position, adjBound ) ) {
			if( crumble_index < ( crumbleState->count - crumbleState->timer_count ) ) {
				int32 target_index = crumbleState->count - crumbleState->timer_count - 1;
				
				rect temp_bound = crumbleState->bound[ crumble_index ];
				crumbleState->bound[ crumble_index ] = crumbleState->bound[ target_index ];
				crumbleState->bound[  target_index ] = temp_bound;
				
				crumbleState->timer[	crumbleState->timer_count++ ] = 0.0f;
				Assert( crumbleState->timer_count < CRUMBLE__MAX_TIMER_COUNT );
			} else {
				crumble_index++;
			}
		} else {
			crumble_index++;
		}
	}
}

internal void
testTimer( CRUMBLE_STATE * crumbleState ) {
	int32 newCount = 0;
	for( int32 crumble_index = 0; crumble_index < crumbleState->timer_count; crumble_index++ ) {
		flo32 time = crumbleState->timer[ crumble_index ];
		
		if( time < 0.5f ) {
			crumbleState->timer[ newCount++ ] = time;
		}
	}
	crumbleState->count -= ( crumbleState->timer_count - newCount );
	crumbleState->timer_count = newCount;
}

internal void
draw( BACKBUFFER * backbuffer, VIEWPORT * camera, CRUMBLE_STATE * crumbleState ) {
	for( int32 crumble_index = 0; crumble_index < crumbleState->count; crumble_index++ ) {
		rect    bound = crumbleState->bound[ crumble_index ];
		rect adjBound = rectMM( transformToView( camera, bound.min ), transformToView( camera, bound.max ) );
		drawBox( backbuffer, adjBound, 0xFF000000, 0xFFFF007F );
	}
}