
#define APP_width      ( 1240 )
#define APP_height     (  760 )

struct BACKBUFFER {
	int32 width;
	int32 height;
	uint32 memory[ APP_width * APP_height ];
};

struct VIEWPORT {
	vec2  position;
	vec2  screenCenter;
};

internal void
clearBackbuffer( BACKBUFFER * backbuffer, uint32 color ) {
	uint32 pixel_count = backbuffer->width * backbuffer->height;
	uint32 * pixel = ( uint32 * )backbuffer->memory;
	for( uint32 counter = 0; counter < pixel_count; counter++ ) {
		*pixel++ = color;
	}
}

internal void
drawRect( BACKBUFFER * backbuffer, rect bound, uint32 color ) {
	int32 xMin = ( int32 )maxValue( bound.left,   0.0f );
	int32 yMin = ( int32 )maxValue( bound.bottom, 0.0f );
	int32 xMax = ( int32 )minValue( bound.right,  ( flo32 )backbuffer->width );
	int32 yMax = ( int32 )minValue( bound.top,    ( flo32 )backbuffer->height );
	
	int32 backbuffer_pitch = backbuffer->width;
	
	uint32 * backbuffer_row = ( uint32 * )backbuffer->memory + yMin * backbuffer_pitch + xMin;
	for( int32 y = yMin; y < yMax; y++ ) {
		uint32 * pixel = backbuffer_row;
		for( int32 x = xMin; x < xMax; x++ ) {
			*pixel++ = color;
		}
		backbuffer_row += backbuffer_pitch;
	}
}

internal void
drawRectOutline( BACKBUFFER * backbuffer, rect bound, uint32 color ) {
	flo32 width  = getWidth(  bound );
	flo32 height = getHeight( bound );
	
	drawRect( backbuffer, Rect( bound.left,         bound.bottom, bound.left + 1.0f, bound.top ), color );
	drawRect( backbuffer, Rect( bound.right - 1.0f, bound.bottom, bound.right,       bound.top ), color );
	drawRect( backbuffer, Rect( bound.left, bound.bottom,     bound.right, bound.bottom + 1.0f ), color );
	drawRect( backbuffer, Rect( bound.left, bound.top - 1.0f, bound.right, bound.top           ), color );
}

internal void
drawBox( BACKBUFFER * backbuffer, rect bound, uint32 outline_color, uint32 background_color ) {
	drawRect(        backbuffer, bound, background_color );
	drawRectOutline( backbuffer, bound, outline_color    );
}

internal vec2
transformToView( VIEWPORT * view, vec2 point ) {
	vec2 result = ( point - view->position ) + view->screenCenter;
	return result;
}

internal rect
transformToView( VIEWPORT * view, rect bound ) {
	rect result = rectMM( transformToView( view, bound.min ), transformToView( view, bound.max ) );
	return result;
}

internal void
drawRect( BACKBUFFER * backbuffer, VIEWPORT * view, rect bound, uint32 color ) {
	rect adjBound = transformToView( view, bound );
	drawRect( backbuffer, adjBound, color );
}

internal void
drawRectOutline( BACKBUFFER * backbuffer, VIEWPORT * view, rect bound, uint32 color ) {
	rect adjBound = transformToView( view, bound );
	drawRectOutline( backbuffer, adjBound, color );
}

internal void
drawBox( BACKBUFFER * backbuffer, VIEWPORT * view, rect bound, uint32 outline_color, uint32 fill_color ) {
	rect adjBound = transformToView( view, bound );
	drawBox( backbuffer, adjBound, outline_color, fill_color );
}

internal void
drawChar( BACKBUFFER * backbuffer, CHAR_TEXTURE * char_texture, vec2 position ) {
	rect bound = Rect( position.x, position.y, position.x + ( flo32 )char_texture->width, position.y + ( flo32 )char_texture->height );
	
	int32 xMin = maxValue( ( int32 )bound.left,   0 );
	int32 yMin = maxValue( ( int32 )bound.bottom, 0 );
	int32 xMax = minValue( ( int32 )bound.right,  backbuffer->width  );
	int32 yMax = minValue( ( int32 )bound.top,    backbuffer->height );
	
	int32 src_x = ( int32 )maxValue( -bound.left,   0.0f );
	int32 src_y = ( int32 )maxValue( -bound.bottom, 0.0f );
	
	int32 backbuffer_pitch = backbuffer->width;
	int32 texture_pitch    = char_texture->width;
	
	uint32 * backbuffer_row = ( uint32 * )backbuffer->memory + yMin * backbuffer_pitch + xMin;
	uint8  * texture_row    = ( uint8  * )char_texture->memory + src_y * texture_pitch + src_x;
	uint32 mask = 0x000000FF;
	
	for( int32 y = yMin; y < yMax; y++ ) {
		uint32 * dest = backbuffer_row;
		uint8  * src  = texture_row;
		
		__m128i _mask    = _mm_set1_epi32( 0x000000FF );
		__m128  _one     = _mm_set1_ps( 1.0f );
		__m128  _inv255  = _mm_set1_ps( 1.0f / 255.0f );
		__m128i _xMax    = _mm_set1_epi32( xMax );
		
		for( int32 x = xMin; x < xMax; x += 4 ) {
			uint32   texel = *( ( uint32  * )src  );
			__m128i _texeli = _mm_set_epi32( ( texel >> 24 ) & mask, ( texel >> 16 ) & mask, ( texel >> 8 ) & mask, ( texel >> 0 ) & mask );
			__m128i _pixeli = _mm_loadu_si128( ( __m128i * )dest );
			
			__m128 _texel = _mm_cvtepi32_ps( _texeli );
			
			__m128 _pixelA = _mm_cvtepi32_ps(                _mm_srli_epi32( _pixeli, 24 )          );
			__m128 _pixelR = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _pixeli, 16 ), _mask ) );
			__m128 _pixelG = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _pixeli,  8 ), _mask ) );
			__m128 _pixelB = _mm_cvtepi32_ps( _mm_and_si128(                 _pixeli      , _mask ) );
			
			__m128 _alpha = _mm_mul_ps( _texel, _inv255 );
			__m128 _OneMinusAlpha = _mm_sub_ps( _one, _alpha );
			
			_pixelA = _one;
			_pixelR = _mm_mul_ps( _OneMinusAlpha, _pixelR );
			_pixelG = _mm_mul_ps( _OneMinusAlpha, _pixelG );
			_pixelB = _mm_mul_ps( _OneMinusAlpha, _pixelB );
			
			__m128i _packedColor =               _mm_slli_epi32( _mm_cvtps_epi32( _pixelA ), 24 );
			        _packedColor = _mm_or_si128( _mm_slli_epi32( _mm_cvtps_epi32( _pixelR ), 16 ), _packedColor );
			        _packedColor = _mm_or_si128( _mm_slli_epi32( _mm_cvtps_epi32( _pixelG ),  8 ), _packedColor );
			        _packedColor = _mm_or_si128(                 _mm_cvtps_epi32( _pixelB ),       _packedColor );
			
			__m128i _writeMask = _mm_cmplt_epi32( _mm_set_epi32( x + 3, x + 2, x + 1, x ), _xMax );
			__m128i _maskedColor = _mm_and_si128( _packedColor, _writeMask );
			        _maskedColor = _mm_or_si128(  _maskedColor, _mm_andnot_si128( _writeMask, _pixeli ) );
			
			_mm_storeu_si128( ( __m128i * )dest, _maskedColor );
			
			dest += 4;
			src  += 4;
		}
		
		backbuffer_row += backbuffer_pitch;
		texture_row    += texture_pitch;
	}
}

internal flo32
getWidth( FONT * font, char * string ) {
	flo32 result = 0.0f;
	
	char * at = string;
	
	while( *at ) {
		if( *at == ' ' ) {
			result += font->advanceWidth_space;
		} else {
			uint8 charA  = *at;
			int32 indexA = charA - FONT_START_CHAR;
			
			FONT_CHAR * fontChar = font->alphabet + indexA;
			result += fontChar->advanceWidth;
			
			uint8 charB = *( at + 1 );
			if( charB ) {
				int32 indexB    = charB - FONT_START_CHAR;
				flo32 kernValue = ( flo32 )font->kernValue[ indexA * FONT_CHAR_COUNT + indexB ];
				result += kernValue;
			}
		}
		at++;
	}
	return result;
}

internal void
drawString( BACKBUFFER * backbuffer, FONT * font, char * string, vec2 position ) {
	vec2 atPos = position;
	char * at = string;
	while( *at ) {
		if( *at == ' ' ) {
			atPos.x += font->advanceWidth_space;
		} else {
			uint8 charA  = *at;
			int32 indexA = charA - FONT_START_CHAR;
			
			FONT_CHAR * fontChar        = font->alphabet     + indexA;
			CHAR_TEXTURE * char_texture = font->char_texture + indexA;
			
			drawChar( backbuffer, char_texture, atPos + fontChar->offset );
			atPos.x += fontChar->advanceWidth;
			
			uint8 charB = *( at + 1 );
			if( charB ) {
				int32 indexB    = charB - FONT_START_CHAR;
				flo32 kernValue = ( flo32 )font->kernValue[ indexA * FONT_CHAR_COUNT + indexB ];
				atPos.x += kernValue;
			}
		}
		at++;
	}
}

internal void
drawString( BACKBUFFER * backbuffer, FONT * font, char * string, VIEWPORT * camera, vec2 position ) {
	vec2 adjPos = transformToView( camera, position );
	drawString( backbuffer, font, string, adjPos );
}

internal void
drawStringCentered( BACKBUFFER * backbuffer, FONT * font, char * string, vec2 position ) {
	flo32 string_width  = getWidth( font, string );
	flo32 string_height = font->advanceHeight;
	
	vec2 adjPos = position + Vec2( string_width, string_height ) * -0.5f + Vec2( 0.0f, font->descent + font->lineGap );
	drawString( backbuffer, font, string, adjPos );
}

internal void
drawCircle( BACKBUFFER * backbuffer, vec2 center, flo32 radius, uint32 color ) {
	flo32 radiusSq = squareValue( radius );
	
	int32 yMin = maxValue( ( int32 )( center.y - radius ), 0 );
	int32 yMax = minValue( ( int32 )( center.y + radius ), backbuffer->height );
	
	int32 backbuffer_pitch = backbuffer->width;
	
	uint32 * backbuffer_row = ( uint32 * )backbuffer->memory + yMin * backbuffer_pitch;
	for( int32 y = yMin; y < yMax; y++ ) {
		flo32 t = sqrtf( radiusSq - squareValue( center.y - ( flo32 )y ) );
		
		int32 xMin = maxValue( ( int32 )( center.x - t + 0.5f ), 0 );
		int32 xMax = minValue( ( int32 )( center.x + t + 0.5f ), backbuffer->width );
		
		uint32 * pixel = backbuffer_row + xMin;
		for( int32 x = xMin; x < xMax; x++ ) {
			*pixel++ = color;
		}
		backbuffer_row += backbuffer_pitch;
	}
}

internal void
drawPixel( BACKBUFFER * backbuffer, int32 x, int32 y, uint32 color ) {
	if( x >= 0 && x < backbuffer->width && y >= 0 && y < backbuffer->height ) {
		uint32 * pixel = backbuffer->memory + y * ( int32 )backbuffer->width + x;
		*pixel = color;
	}
}

internal void
drawCircleOutline( BACKBUFFER * backbuffer, vec2 center, flo32 radius, uint32 color ) {
	int32 x0 = ( int32 )center.x;
	int32 y0 = ( int32 )center.y;
	
	int32 x = ( int32 )radius;
	int32 y = 0;
	int32 decisionOver2 = 1 - x;
	
	while( y <= x ) {
		drawPixel( backbuffer,  x + x0,  y + y0, color );
		drawPixel( backbuffer,  y + x0,  x + y0, color );
		drawPixel( backbuffer, -x + x0,  y + y0, color );
		drawPixel( backbuffer, -y + x0,  x + y0, color );
		drawPixel( backbuffer, -x + x0, -y + y0, color );
		drawPixel( backbuffer, -y + x0, -x + y0, color );
		drawPixel( backbuffer,  x + x0, -y + y0, color );
		drawPixel( backbuffer,  y + x0, -x + y0, color );
		y++;
		if( decisionOver2 <= 0 ) {
			decisionOver2 += 2 * y + 1;
		} else {
			x--;
			decisionOver2 += 2 * ( y - x ) + 1;
		}
	}
}

internal void
drawCircle( BACKBUFFER * backbuffer, VIEWPORT * camera, vec2 center, flo32 radius, uint32 outline_color, uint32 fill_color ) {
	vec2  adjCenter = transformToView( camera, center );
	drawCircle(        backbuffer, adjCenter, radius, fill_color    );
	drawCircleOutline( backbuffer, adjCenter, radius, outline_color );
}

internal void
drawTriangle( BACKBUFFER * backbuffer, rect bound, uint32 color ) {
	flo32 invHeight = 1.0f / getHeight( bound );
	flo32 halfWidth = getWidth(  bound ) * 0.5f;
	flo32 centerX   = ( bound.left + bound.right ) * 0.5f;
	
	int32 backbuffer_pitch = backbuffer->width;
	
	int32 yMin = maxValue( ( int32 )bound.bottom, 0 );
	int32 yMax = minValue( ( int32 )bound.top,    backbuffer->height );
	
	uint32 * backbuffer_row = ( uint32 * )backbuffer->memory + yMin * backbuffer_pitch;
	for( int32 y = yMin; y < yMax; y++ ) {
		flo32 t = ( bound.top - ( flo32 )y ) * invHeight * halfWidth;
		int32 xMin = maxValue( ( int32 )( centerX - t + 0.5f ), 0 );
		int32 xMax = minValue( ( int32 )( centerX + t + 0.5f ), backbuffer->width );
		
		uint32 * pixel = backbuffer_row + xMin;
		for( int32 x = xMin; x < xMax; x++ ) {
			*pixel++ = color;
		}
		backbuffer_row += backbuffer_pitch;
	}
}

internal void
drawTriangle( BACKBUFFER * backbuffer, VIEWPORT * view, rect bound, uint32 color ) {
	rect adjBound = rectMM( transformToView( view, bound.min ), transformToView( view, bound.max ) );
	drawTriangle( backbuffer, adjBound, color );
}

internal void
drawLine( BACKBUFFER * backbuffer, vec2 startPoint, vec2 endPoint, uint32 color ) {
	boo32 drawLine = false;
	
	rect screen_bound = Rect( 0.0f, 0.0f, ( flo32 )backbuffer->width, ( flo32 )backbuffer->height );
	if( isInBound( startPoint, screen_bound ) && isInBound( endPoint, screen_bound ) ) {
		drawLine = true;
	} else {
		vec2 vector = endPoint - startPoint;
		if( vector.x != 0.0f ) {
			flo32 t_left  = ( screen_bound.left - startPoint.x ) / vector.x;
			if( ( t_left > 0.0f ) && ( t_left < 1.0f ) ) {
				flo32 intersectY = startPoint.y + vector.y * t_left;
				if( ( intersectY >= screen_bound.bottom ) && ( intersectY <= screen_bound.top ) ) {
					drawLine = true;
					if( startPoint.x < screen_bound.left ) {
						startPoint = Vec2( screen_bound.left, intersectY );
					} else {
						endPoint   = Vec2( screen_bound.left, intersectY );
					}
					vector = endPoint - startPoint;
				}
			}
			
			flo32 t_right = ( screen_bound.right - startPoint.x ) / vector.x;
			if( ( t_right > 0.0f ) && ( t_right < 1.0f ) ) {
				flo32 intersectY = startPoint.y + vector.y * t_right;
				if( ( intersectY >= screen_bound.bottom ) && ( intersectY <= screen_bound.top ) ) {
					drawLine = true;
					if( startPoint.x > screen_bound.right ) {
						startPoint = Vec2( screen_bound.right, intersectY );
					} else {
						endPoint   = Vec2( screen_bound.right, intersectY );
					}
					vector = endPoint - startPoint;
				}
			}
		}
		if( vector.y != 0.0f ) {
			flo32 t_bottom  = ( screen_bound.bottom - startPoint.y ) / vector.y;
			if( ( t_bottom > 0.0f ) && ( t_bottom < 1.0f ) ) {
				flo32 intersectX = startPoint.x + vector.x * t_bottom;
				if( ( intersectX >= screen_bound.left ) && ( intersectX <= screen_bound.right ) ) {
					drawLine = true;
					if( startPoint.y < screen_bound.bottom ) {
						startPoint = Vec2( intersectX, screen_bound.bottom );
					} else {
						endPoint   = Vec2( intersectX, screen_bound.bottom );
					}
					vector = endPoint - startPoint;
				}
			}
			
			flo32 t_top = ( screen_bound.top - startPoint.y ) / vector.y;
			if( ( t_top > 0.0f ) && ( t_top < 1.0f ) ) {
				flo32 intersectX = startPoint.x + vector.x * t_top;
				if( ( intersectX >= screen_bound.left ) && ( intersectX <= screen_bound.right ) ) {
					drawLine = true;
					if( startPoint.y > screen_bound.top ) {
						startPoint = Vec2( intersectX, screen_bound.top );
					} else {
						endPoint   = Vec2( intersectX, screen_bound.top );
					}
					vector = endPoint - startPoint;
				}
			}
		}
	}
	
	if( drawLine ) {
		int32 xMin = ( int32 )minValue( startPoint.x, endPoint.x );
		int32 xMax = ( int32 )maxValue( startPoint.x, endPoint.x );
		int32 yMin = ( int32 )minValue( startPoint.y, endPoint.y );
		int32 yMax = ( int32 )maxValue( startPoint.y, endPoint.y );
		
		vec2  vector     = endPoint - startPoint;
		int32 step_count = ( int32 )getLength( vector );
		vec2  step       = getNormal( vector );
		vec2  position   = startPoint;
		uint32 * memory = ( uint32 * )backbuffer->memory;
		
		for( int32 counter = 0; counter < step_count; counter++ ) {
			int32 posX = ( int32 )position.x;
			int32 posY = ( int32 )position.y;
			
			if( ( posX >= xMin ) && ( posX < xMax ) && ( posY >= yMin ) && ( posY < yMax ) ) {
				memory[ posY * backbuffer->width + posX ] = color;
			}
			position += step;
		}
	}
}

internal void
drawLine( BACKBUFFER * backbuffer, VIEWPORT * view, vec2 startPoint, vec2 endPoint, uint32 color ) {
	drawLine( backbuffer, transformToView( view, startPoint ), transformToView( view, endPoint ), color );
}
