
enum KEY_STATE_CODE {
	keyCode_null,
	
	keyCode_backspace,
	keyCode_shift,
	keyCode_control,
	keyCode_alt,
	keyCode_capsLock,
	keyCode_escape,
	keyCode_pageUp,
	keyCode_pageDown,
	keyCode_end,
	keyCode_home,
	keyCode_left,
	keyCode_right,
	keyCode_up,
	keyCode_down,
	keyCode_insert,
	keyCode_delete,
	keyCode_enter,
	keyCode_tab,
	keyCode_space,
	keyCode_0,
	keyCode_1,
	keyCode_2,
	keyCode_3,
	keyCode_4,
	keyCode_5,
	keyCode_6,
	keyCode_7,
	keyCode_8,
	keyCode_9,
	keyCode_a,
	keyCode_b,
	keyCode_c,
	keyCode_d,
	keyCode_e,
	keyCode_f,
	keyCode_g,
	keyCode_h,
	keyCode_i,
	keyCode_j,
	keyCode_k,
	keyCode_l,
	keyCode_m,
	keyCode_n,
	keyCode_o,
	keyCode_p,
	keyCode_q,
	keyCode_r,
	keyCode_s,
	keyCode_t,
	keyCode_u,
	keyCode_v,
	keyCode_w,
	keyCode_x,
	keyCode_y,
	keyCode_z,
	keyCode_semicolon,
	keyCode_equal,
	keyCode_comma,
	keyCode_dash,
	keyCode_period,
	keyCode_forwardSlash,
	keyCode_tilde,
	keyCode_openBracket,
	keyCode_backslash,
	keyCode_closeBracket,
	keyCode_quote,
	keyCode_num0,
	keyCode_num1,
	keyCode_num2,
	keyCode_num3,
	keyCode_num4,
	keyCode_num5,
	keyCode_num6,
	keyCode_num7,
	keyCode_num8,
	keyCode_num9,
	
	keyCode_count,
};

struct KEY_STATE {
	KEY_STATE_CODE keyCode;
	boo32      isDown;
	boo32 shiftIsDown;
};

internal KEY_STATE
KeyState( KEY_STATE_CODE keyCode, boo32 isDown, boo32 shiftIsDown ) {
	KEY_STATE result = {};
	result.keyCode     = keyCode;
	result.isDown      = isDown;
	result.shiftIsDown = shiftIsDown;
	return result;
}

#define KEYBOARD_STATE__MAX_EVENT_COUNT  ( 16 )
struct KEYBOARD_STATE {
	KEY_STATE event[ KEYBOARD_STATE__MAX_EVENT_COUNT ];
	int32 event_count;
	
	boo32 shiftIsDown;
	boo32 controlIsDown;
};

internal boo32
wasPressed( KEYBOARD_STATE * keyboard, KEY_STATE_CODE keyCode ) {
	boo32 result = false;
	for( int32 event_index = 0; event_index < keyboard->event_count; event_index++ ) {
		KEY_STATE event = keyboard->event[ event_index ];
		if( ( event.keyCode == keyCode ) && ( event.isDown ) ) {
			result = true;
		}
	}
	return result;
}

internal boo32
wasReleased( KEYBOARD_STATE * keyboard, KEY_STATE_CODE keyCode ) {
	boo32 result = false;
	for( int32 event_index = 0; event_index < keyboard->event_count; event_index++ ) {
		KEY_STATE event = keyboard->event[ event_index ];
		if( ( event.keyCode == keyCode ) && ( !event.isDown ) ) {
			result = true;
		}
	}
	return result;
}

internal void
addKeyEvent( KEYBOARD_STATE * keyboard, KEY_STATE_CODE keyCode, boo32 isDown, boo32 shiftIsDown = false ) {
	Assert( keyboard->event_count < KEYBOARD_STATE__MAX_EVENT_COUNT );
	keyboard->event[ keyboard->event_count++ ] = KeyState( keyCode, isDown, shiftIsDown );
}
