
struct SAWBLADE {
	vec2 position;
	flo32 radius;
};

struct MOVING_STATE {
	vec2 velocity;
	flo32 timer;
	flo32 halfCycleTime;
};

struct ROTATING_STATE {
	vec2 center;
	flo32 timer;
	flo32 rotationTime;
	flo32 armRadius;
};

#define SAWBLADE__MAX_FLYING_COUNT  ( 8 )
struct LAUNCH_STATE {
	vec2 position;
	vec2 velocity;
	
	flo32 launch_timer;
	flo32 launchTime;
	flo32 flightTime;
	
	int32 count;
	int32 baseIndex;
	int32 currIndex;
	
	flo32 flight_timer[ SAWBLADE__MAX_FLYING_COUNT ];
};

#define SAWBLADE__MAX_COUNT               ( 64 )
#define SAWBLADE__MAX_MOVING_COUNT        ( 12 )
#define SAWBLADE__MAX_ROTATING_COUNT      ( 12 )
#define SAWBLADE__MAX_LAUNCH_STATE_COUNT  ( 12 )
struct SAWBLADE_STATE {
	int32 moving_baseIndex;
	int32 moving_count;
	MOVING_STATE moving[ SAWBLADE__MAX_MOVING_COUNT ];
	
	int32 rotating_baseIndex;
	int32 rotating_count;
	ROTATING_STATE rotating[ SAWBLADE__MAX_ROTATING_COUNT ];
	
	int32 launch_count;
	LAUNCH_STATE launch[ SAWBLADE__MAX_LAUNCH_STATE_COUNT ];
	
	int32 count;
	SAWBLADE sawblade[ SAWBLADE__MAX_COUNT ];
};

internal SAWBLADE
SawBlade( vec2 position, flo32 radius ) {
	SAWBLADE result = {};
	result.position = position;
	result.radius   = radius;
	return result;
}

internal MOVING_STATE
MovingState( vec2 velocity, flo32 halfCycleTime ) {
	MOVING_STATE result = {};
	result.velocity      = velocity;
	result.halfCycleTime = halfCycleTime;
	return result;
}

internal ROTATING_STATE
RotatingState( vec2 center, flo32 rotationTime, flo32 armRadius ) {
	ROTATING_STATE result = {};
	result.center       = center;
	result.rotationTime = rotationTime;
	result.armRadius    = armRadius;
	return result;
}

internal void
updateMoving( SAWBLADE_STATE * sawbladeState, flo32 dt ) {
	for( int32 sawblade_index = 0; sawblade_index < sawbladeState->moving_count; sawblade_index++ ) {
		SAWBLADE *   sawblade = sawbladeState->sawblade + sawbladeState->moving_baseIndex + sawblade_index;
		MOVING_STATE * moving = sawbladeState->moving + sawblade_index;
		
		sawblade->position += moving->velocity * dt;
		moving->timer += dt;
		if( moving->timer >= moving->halfCycleTime ) {
			moving->timer   -=  moving->halfCycleTime;
			moving->velocity = -moving->velocity;
		}
	}
}

internal void
updateRotating( SAWBLADE_STATE * sawbladeState, flo32 dt ) {
	for( int32 sawblade_index = 0; sawblade_index < sawbladeState->rotating_count; sawblade_index++ ) {
		SAWBLADE * sawblade       = sawbladeState->sawblade + sawbladeState->rotating_baseIndex + sawblade_index;
		ROTATING_STATE * rotating = sawbladeState->rotating + sawblade_index;
		
		rotating->timer += dt;
		
		flo32 radians = rotating->timer / rotating->rotationTime * 2.0f * PI;
		sawblade->position = rotating->center + Vec2( cosf( radians ), sinf( radians ) ) * rotating->armRadius;
	}
}

internal void
updateFlying( SAWBLADE_STATE * sawbladeState, flo32 dt ) {
	for( int32 launch_index = 0; launch_index < sawbladeState->launch_count; launch_index++ ) {
		LAUNCH_STATE * launchState = sawbladeState->launch + launch_index;
		
		launchState->launch_timer += dt;
		if( launchState->launch_timer >= launchState->launchTime ) {
			launchState->launch_timer -= launchState->launchTime;
			sawbladeState->sawblade[ launchState->baseIndex + launchState->currIndex ].position = launchState->position;
			launchState->flight_timer[ launchState->currIndex ] = 0.0f;
			launchState->currIndex = ( ( launchState->currIndex + 1 ) % launchState->count );
		}
		for( int32 sawblade_index = 0; sawblade_index < launchState->count; sawblade_index++ ) {
			sawbladeState->sawblade[ launchState->baseIndex + sawblade_index ].position += ( launchState->velocity * dt );
			launchState->flight_timer[ sawblade_index ] += dt;
			if( launchState->flight_timer[ sawblade_index ] >= launchState->flightTime ) {
				launchState->flight_timer[ sawblade_index ] = -1000000.0f;
				sawbladeState->sawblade[ launchState->baseIndex + sawblade_index ].position = Vec2( 0.0f, -1000000.0f );
			}
		}
	}
}

internal void
draw( BACKBUFFER * backbuffer, VIEWPORT * camera, SAWBLADE_STATE * sawbladeState ) {
	if( sawbladeState->launch_count > 0 ) {
		int32 flying_minIndex = sawbladeState->launch[ 0 ].baseIndex - 1;
		int32 flying_maxIndex = sawbladeState->launch[ sawbladeState->launch_count - 1 ].baseIndex + sawbladeState->launch[ sawbladeState->launch_count - 1 ].count;
		
		for( int32 sawblade_index = 0; sawblade_index < flying_minIndex; sawblade_index++ ) {
			SAWBLADE * sawblade = sawbladeState->sawblade + sawblade_index;	
			drawCircle( backbuffer, camera, sawblade->position, sawblade->radius + 4.0f, 0xFF000000, 0xFFC8C8C8 );
		}
		for( int32 sawblade_index = flying_minIndex; sawblade_index < flying_maxIndex; sawblade_index++ ) {
			SAWBLADE * sawblade = sawbladeState->sawblade + sawblade_index;
			drawCircle( backbuffer, camera, sawblade->position, sawblade->radius + 2.0f, 0xFF000000, 0xFF282828 );
		}
		for( int32 sawblade_index = flying_maxIndex; sawblade_index < sawbladeState->count; sawblade_index++ ) {
			SAWBLADE * sawblade = sawbladeState->sawblade + sawblade_index;
			drawCircle( backbuffer, camera, sawblade->position, sawblade->radius + 4.0f, 0xFF000000, 0xFFC8C8C8 );
		}
	} else {
		for( int32 sawblade_index = 0; sawblade_index < sawbladeState->count; sawblade_index++ ) {
			SAWBLADE * sawblade = sawbladeState->sawblade + sawblade_index;
			drawCircle( backbuffer, camera, sawblade->position, sawblade->radius + 4.0f, 0xFF000000, 0xFFC8C8C8 );
		}
	}
	for( int32 sawblade_index = 0; sawblade_index < sawbladeState->rotating_count; sawblade_index++ ) {
		SAWBLADE * sawblade       = sawbladeState->sawblade + sawbladeState->rotating_baseIndex + sawblade_index;
		ROTATING_STATE * rotating = sawbladeState->rotating + sawblade_index;
		
		drawLine( backbuffer, camera, sawblade->position, rotating->center, 0xFF000000 );
	}
}