
struct SPLATTER {
	flo32 timeToLive;
	vec2  position[ 16 ];
	vec2  velocity[ 16 ];
};

#define MAX_SPLATTER_COUNT 256
struct SPLATTER_BUFFER {
	uint32 count;
	uint32 startIndex;
	uint32 endIndex;
	SPLATTER splatter[ MAX_SPLATTER_COUNT ];
};

internal void
pushNewSplatter( SPLATTER_BUFFER * splatterBuffer, vec2 position, flo32 dt ) {
	if( splatterBuffer->count < MAX_SPLATTER_COUNT ) {
		SPLATTER * currentSplatter = splatterBuffer->splatter + splatterBuffer->endIndex;
		currentSplatter->timeToLive = 1.5f;
		for( uint32 chunk_index = 0; chunk_index < 16; chunk_index++ ) {
			currentSplatter->velocity[ chunk_index ] = Vec2( flo32( rand() % 400 - 200 ), flo32( rand() % 400 - 100 ) );
			currentSplatter->position[ chunk_index ] = position + currentSplatter->velocity[ chunk_index ] * dt;
		}
		splatterBuffer->endIndex = ( splatterBuffer->endIndex + 1 ) % MAX_SPLATTER_COUNT;
		splatterBuffer->count++;
	}
}

internal void
updateSplatter( SPLATTER_BUFFER * splatterBuffer, flo32 dt ) {
	for( uint32 counter = 0; counter < splatterBuffer->count; counter++ ) {
		uint32 index = ( splatterBuffer->startIndex + counter ) % MAX_SPLATTER_COUNT;
		SPLATTER * splatter = splatterBuffer->splatter + index;
		if( splatter->timeToLive > 0.0f ) {
			splatter->timeToLive -= dt;
			
			vec2 accel = { 0.0f, -800.0f };
			for( uint32 chunk_index = 0; chunk_index < 16; chunk_index++ ) {
				splatter->velocity[ chunk_index ] += accel * dt;
				splatter->position[ chunk_index ] += accel * 0.5f * dt * dt + splatter->velocity[ chunk_index ] * dt;
			}
		} else {
			splatterBuffer->startIndex = ( splatterBuffer->startIndex + 1 ) % MAX_SPLATTER_COUNT;
			splatterBuffer->count--;
		}
	}
}

internal void
drawSplatter( BACKBUFFER * backbuffer, VIEWPORT * camera, SPLATTER_BUFFER * splatterBuffer ) {
	for( uint32 counter = 0; counter < splatterBuffer->count; counter++ ) {
		uint32 index = ( splatterBuffer->startIndex + counter ) % MAX_SPLATTER_COUNT;
		
		SPLATTER * splatter = splatterBuffer->splatter + index;
		for( uint32 chunk_index = 0; chunk_index < 16; chunk_index++ ) {
			rect bound = rectCR( splatter->position[ chunk_index ], 4.0f );
			drawBox( backbuffer, camera, bound, 0xFF000000, 0xFFFF0000 );
		}
	}
}