
//----------
// vec2 functions
//----------

union vec2 {
	struct {
		flo32 x;
		flo32 y;
	};
	flo32 elem[2];
};

inline vec2 Vec2( flo32 a, flo32 b ) { vec2 result = { a, b }; return result; }
inline vec2 operator+( vec2 a, vec2 b ) { vec2 result = { a.x + b.x, a.y + b.y }; return result; }
inline vec2 operator-( vec2 a, vec2 b ) { vec2 result = { a.x - b.x, a.y - b.y }; return result; }
inline vec2 operator-( vec2 a ) { vec2 result = { -a.x, -a.y }; return result; }
inline vec2 operator*( vec2 a, flo32 f ) { vec2 result = { a.x * f, a.y * f }; return result; }
inline vec2 operator*( flo32 f, vec2 a ) { vec2 result = a * f; return result; }
inline vec2 operator*( vec2 a, vec2 b ) { vec2 result = { a.x * b.x, a.y * b.y }; return result; }
inline vec2 operator/( vec2 a, flo32 f ) { vec2 result = {}; if( f != 0.0f ) { result = Vec2( a.x / f, a.y / f ); } return result; }
inline vec2 operator/( vec2 a, vec2 b ) { vec2 result = { ( b.x != 0.0f ) ? a.x / b.x : 0.0f, ( b.y != 0.0f ) ? a.y / b.y : 0.0f }; return result; }
inline vec2 & operator+=( vec2 & a, vec2 b ) { a = a + b; return a; }
inline vec2 & operator-=( vec2 & a, vec2 b ) { a = a - b; return a; }
inline vec2 & operator*=( vec2 & a, flo32 f ) { a = a * f; return a; }
inline vec2 & operator/=( vec2 & a, flo32 f ) { a = ( f != 0.0f ) ? a / f : Vec2( 0.0f, 0.0f );	return a; }
inline boo32 operator==( vec2 a, vec2 b ) { boo32 result = ( a.x == b.x ) && ( a.y == b.y ); return result; }
inline boo32 operator!=( vec2 a, vec2 b ) { boo32 result = ( a.x != b.x ) || ( a.y != b.y ); return result; }
inline flo32 getLengthSq( vec2 a ) { flo32 result = ( a.x * a.x ) + ( a.y * a.y ); return result; }
inline flo32 getLength( vec2 a ) { flo32 result = sqrtf( getLengthSq( a ) ); return result; }
inline vec2 getNormal( vec2 a ) { flo32 denom = getLength( a ); vec2 result = a / denom; return result; }
inline vec2 getPerp( vec2 a ) { vec2 result = { -a.y, a.x }; return result; }
inline flo32 dot( vec2 a, vec2 b ) { flo32 result = a.x * b.x + a.y * b.y; return result; }

//----------
// rect functions
//----------

union rect {
	struct {
		vec2 min;
		vec2 max;
	};
	struct {
		flo32 left;
		flo32 bottom;
		flo32 right;
		flo32 top;
	};
	flo32 elem[4];
};

inline rect Rect( flo32 left, flo32 bottom, flo32 right, flo32 top ) { rect result = { left, bottom, right, top }; return result; }
inline flo32 getWidth(  rect r ) { flo32 result = ( r.right - r.left ); return result; }
inline flo32 getHeight( rect r ) { flo32 result = ( r.top - r.bottom ); return result; }
inline vec2 getTL( rect r ) { vec2 result = Vec2( r.left,  r.top    ); return result; }
inline vec2 getTR( rect r ) { vec2 result = Vec2( r.right, r.top    ); return result; }
inline vec2 getBL( rect r ) { vec2 result = Vec2( r.left,  r.bottom ); return result; }
inline vec2 getBR( rect r ) { vec2 result = Vec2( r.right, r.bottom ); return result; }
inline vec2 getTC( rect r ) { vec2 result = Vec2( ( r.left + r.right ) * 0.5f, r.top    ); return result; }
inline vec2 getBC( rect r ) { vec2 result = Vec2( ( r.left + r.right ) * 0.5f, r.bottom ); return result; }
inline vec2 getLC( rect r ) { vec2 result = Vec2( r.left,  ( r.bottom + r.top ) * 0.5f );  return result; }
inline vec2 getRC( rect r ) { vec2 result = Vec2( r.right, ( r.bottom + r.top ) * 0.5f );  return result; }
inline rect rectTLD( vec2 topLeft,     vec2 dim ) { rect result = Rect( topLeft.x,             topLeft.y - dim.y,  topLeft.x + dim.x,    topLeft.y             ); return result; }
inline rect rectTRD( vec2 topRight,    vec2 dim ) { rect result = Rect( topRight.x - dim.x,    topRight.y - dim.y, topRight.x,           topRight.y            ); return result; }
inline rect rectBLD( vec2 bottomLeft,  vec2 dim ) { rect result = Rect( bottomLeft.x,          bottomLeft.y,       bottomLeft.x + dim.x, bottomLeft.y  + dim.y ); return result; }
inline rect rectBRD( vec2 bottomRight, vec2 dim ) { rect result = Rect( bottomRight.x - dim.x, bottomRight.y,      bottomRight.x,        bottomRight.y + dim.y ); return result; }
inline rect rectCD(  vec2 center, vec2 dim ) { vec2 halfDim = dim * 0.5f; rect result = Rect( center.x - halfDim.x, center.y - halfDim.y, center.x + halfDim.x, center.y + halfDim.y ); return result; }
inline rect rectCHD( vec2 center, vec2 halfDim ) { rect result = Rect( center.x - halfDim.x, center.y - halfDim.y, center.x + halfDim.x, center.y + halfDim.y ); return result; }
inline rect rectCR(  vec2 center, flo32 radius ) { rect result = Rect( center.x - radius, center.y - radius, center.x + radius, center.y + radius ); return result; }
inline rect rectMM(  vec2 min, vec2 max ) { rect result = Rect( min.x, min.y, max.x, max.y ); return result; }
inline rect addDim( rect r, flo32 x0, flo32 y0, flo32 x1, flo32 y1 ) { rect result = Rect( r.left - x0, r.bottom - y0, r.right + x1, r.top + y1 ); return result; }
inline rect addRadius( rect r, flo32 radius ) { rect result = Rect( r.left - radius, r.bottom - radius, r.right + radius, r.top + radius ); return result; }
inline vec2 getCenter( rect r ) { vec2 result = Vec2( ( r.left + r.right ) * 0.5f, ( r.bottom + r.top ) * 0.5f ); return result; }

inline boo32 isInBound( vec2 point, rect bound ) { boo32 result = ( point.x >= bound.left ) && ( point.y >= bound.bottom ) && ( point.x <= bound.right ) && ( point.y <= bound.top ); return result; }