
internal int64
win32_getPerfCount() {
	LARGE_INTEGER perfCounter = {};
	QueryPerformanceCounter( &perfCounter );
	
	int64 result = *( ( int64 * )&perfCounter );
	return result;
}

internal flo32
win32_getMSElapsed( int64 startCounter, int64 endCounter, LARGE_INTEGER perfFrequency ) {
	LARGE_INTEGER counterA = *( ( LARGE_INTEGER * )&startCounter );
	LARGE_INTEGER counterB = *( ( LARGE_INTEGER * )&endCounter );
	flo32 result = 1000.0f * ( ( flo32 )( counterB.QuadPart - counterA.QuadPart ) / ( flo32 )perfFrequency.QuadPart );
	return result;
}

internal FILE_DATA
win32_readFile( MEMORY * memory, char * filename ) {
	FILE_DATA result = {};
	
	HANDLE fileHandle = CreateFile( filename, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0 );
	if( fileHandle != INVALID_HANDLE_VALUE ) {
		LARGE_INTEGER fileSize;
		if( GetFileSizeEx( fileHandle, &fileSize ) ) {
			result.sizeInBytes = ( uint32 )fileSize.QuadPart;
			result.contents = _pushSize_clear( memory, result.sizeInBytes );
			
			DWORD bytesRead;
			if( ReadFile( fileHandle, result.contents, result.sizeInBytes, &bytesRead, 0 ) && ( result.sizeInBytes == bytesRead ) ) {
			} else {
				_popSize( memory, result.sizeInBytes );
				result.contents = 0;
				OutputDebugString( "ERROR. Unable to read file contents.\n" );
			}
			CloseHandle( fileHandle );
		} else {
			OutputDebugString( "ERROR. File does not contain any data.\n" );
		}
	} else {
		char debugStr[128] = {};
		sprintf( debugStr, "ERROR. Unable to open file: %s\n", filename );
		OutputDebugString( debugStr );
	}
	
	return result;
}

internal boo32
win32_writeFile( char * filename, void * data, uint32 size ) {
	boo32 result = false;

	HANDLE fileHandle = CreateFile( filename, GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0 );
	if( fileHandle != INVALID_HANDLE_VALUE ) {
		DWORD bytesWritten;
		if( WriteFile( fileHandle, data, size, &bytesWritten, 0 ) ) {
			result = ( bytesWritten == size );
		}

		CloseHandle( fileHandle );
	} else {
		Assert( !"Unable to write file!" );
	}

	return result;
}

LRESULT CALLBACK
win32_WindowProc( HWND window, uint32 message, WPARAM wParam, LPARAM lParam ) {
	LRESULT result = 0;
	switch( message ) {
		case WM_ACTIVATEAPP: {
		} break;

		case WM_CLOSE: {
		} break;

		case WM_DESTROY: {
		} break;
		
		case WM_SYSKEYDOWN:
		case WM_SYSKEYUP:
		case WM_KEYDOWN:
		case WM_KEYUP: {
			Assert( !"ERROR! Keyboard message was passed directly to WindowProc! All keyboard messages should be handled directly by the game!" );
		} break;
		
		default: {
			result = DefWindowProc( window, message, wParam, lParam );
		} break;
	}
	return result;
}

internal void
win32_processPendingMessages( PLATFORM * platform, HWND windowHandle ) {
	KEYBOARD_STATE * keyboard = &platform->keyboard;
	keyboard->event_count = 0;
	
	MSG message;
	while( PeekMessage( &message, 0, 0, 0, PM_REMOVE ) ) {
		switch( message.message ) {
			case WM_SYSKEYDOWN:
			case WM_SYSKEYUP:
			case WM_KEYDOWN:
			case WM_KEYUP: {
				boo32 wasDown = ( ( message.lParam & ( 1 << 30 ) ) != 0 );
				boo32 isDown  = ( ( message.lParam & ( 1 << 31 ) ) == 0 );
				if( isDown != wasDown ) {
					switch( message.wParam ) {
						case VK_BACK: 		{ addKeyEvent( keyboard, keyCode_backspace, 	isDown ); } break;
						case VK_TAB: 		{ addKeyEvent( keyboard, keyCode_tab, 			isDown ); } break;
						case VK_RETURN:	{ addKeyEvent( keyboard, keyCode_enter, 		isDown ); } break;
						case VK_SHIFT: 	{
							keyboard->shiftIsDown = isDown;
							addKeyEvent( keyboard, keyCode_shift, isDown );
						} break;
						case VK_CONTROL: 	{
							keyboard->controlIsDown = isDown;
							addKeyEvent( keyboard, keyCode_control, isDown );
						} break;
						case VK_MENU:    { addKeyEvent( keyboard, keyCode_alt,      isDown ); } break;
						case VK_CAPITAL: { addKeyEvent( keyboard, keyCode_capsLock, isDown ); } break;
						case VK_ESCAPE:  { addKeyEvent( keyboard, keyCode_escape,   isDown ); } break;
						case VK_SPACE:   { addKeyEvent( keyboard, keyCode_space,    isDown ); } break;
						case VK_PRIOR:   { addKeyEvent( keyboard, keyCode_pageUp,   isDown ); } break;
						case VK_NEXT:    { addKeyEvent( keyboard, keyCode_pageDown, isDown ); } break;
						case VK_END:     { addKeyEvent( keyboard, keyCode_end,      isDown ); } break;
						case VK_HOME:    { addKeyEvent( keyboard, keyCode_home,     isDown ); } break;
						case VK_LEFT:    { addKeyEvent( keyboard, keyCode_left,     isDown ); } break;
						case VK_RIGHT:   { addKeyEvent( keyboard, keyCode_right,    isDown ); } break;
						case VK_UP: 	  { addKeyEvent( keyboard, keyCode_up,       isDown ); } break;
						case VK_DOWN: 	  { addKeyEvent( keyboard, keyCode_down,     isDown ); } break;
						case VK_INSERT:  { addKeyEvent( keyboard, keyCode_insert,   isDown ); } break;
						case VK_DELETE:  { addKeyEvent( keyboard, keyCode_delete,   isDown ); } break;
						case 0x30: { addKeyEvent( keyboard, keyCode_0, isDown, keyboard->shiftIsDown ); } break;
						case 0x31: { addKeyEvent( keyboard, keyCode_1, isDown, keyboard->shiftIsDown ); } break;
						case 0x32: { addKeyEvent( keyboard, keyCode_2, isDown, keyboard->shiftIsDown ); } break;
						case 0x33: { addKeyEvent( keyboard, keyCode_3, isDown, keyboard->shiftIsDown ); } break;
						case 0x34: { addKeyEvent( keyboard, keyCode_4, isDown, keyboard->shiftIsDown ); } break;
						case 0x35: { addKeyEvent( keyboard, keyCode_5, isDown, keyboard->shiftIsDown ); } break;
						case 0x36: { addKeyEvent( keyboard, keyCode_6, isDown, keyboard->shiftIsDown ); } break;
						case 0x37: { addKeyEvent( keyboard, keyCode_7, isDown, keyboard->shiftIsDown ); } break;
						case 0x38: { addKeyEvent( keyboard, keyCode_8, isDown, keyboard->shiftIsDown ); } break;
						case 0x39: { addKeyEvent( keyboard, keyCode_9, isDown, keyboard->shiftIsDown ); } break;
						case 0x41: { addKeyEvent( keyboard, keyCode_a, isDown, keyboard->shiftIsDown ); } break;
						case 0x42: { addKeyEvent( keyboard, keyCode_b, isDown, keyboard->shiftIsDown ); } break;
						case 0x43: { addKeyEvent( keyboard, keyCode_c, isDown, keyboard->shiftIsDown ); } break;
						case 0x44: { addKeyEvent( keyboard, keyCode_d, isDown, keyboard->shiftIsDown ); } break;
						case 0x45: { addKeyEvent( keyboard, keyCode_e, isDown, keyboard->shiftIsDown ); } break;
						case 0x46: { addKeyEvent( keyboard, keyCode_f, isDown, keyboard->shiftIsDown ); } break;
						case 0x47: { addKeyEvent( keyboard, keyCode_g, isDown, keyboard->shiftIsDown ); } break;
						case 0x48: { addKeyEvent( keyboard, keyCode_h, isDown, keyboard->shiftIsDown ); } break;
						case 0x49: { addKeyEvent( keyboard, keyCode_i, isDown, keyboard->shiftIsDown ); } break;
						case 0x4A: { addKeyEvent( keyboard, keyCode_j, isDown, keyboard->shiftIsDown ); } break;
						case 0x4B: { addKeyEvent( keyboard, keyCode_k, isDown, keyboard->shiftIsDown ); } break;
						case 0x4C: { addKeyEvent( keyboard, keyCode_l, isDown, keyboard->shiftIsDown ); } break;
						case 0x4D: { addKeyEvent( keyboard, keyCode_m, isDown, keyboard->shiftIsDown ); } break;
						case 0x4E: { addKeyEvent( keyboard, keyCode_n, isDown, keyboard->shiftIsDown ); } break;
						case 0x4F: { addKeyEvent( keyboard, keyCode_o, isDown, keyboard->shiftIsDown ); } break;
						case 0x50: { addKeyEvent( keyboard, keyCode_p, isDown, keyboard->shiftIsDown ); } break;
						case 0x51: { addKeyEvent( keyboard, keyCode_q, isDown, keyboard->shiftIsDown ); } break;
						case 0x52: { addKeyEvent( keyboard, keyCode_r, isDown, keyboard->shiftIsDown ); } break;
						case 0x53: { addKeyEvent( keyboard, keyCode_s, isDown, keyboard->shiftIsDown ); } break;
						case 0x54: { addKeyEvent( keyboard, keyCode_t, isDown, keyboard->shiftIsDown ); } break;
						case 0x55: { addKeyEvent( keyboard, keyCode_u, isDown, keyboard->shiftIsDown ); } break;
						case 0x56: { addKeyEvent( keyboard, keyCode_v, isDown, keyboard->shiftIsDown ); } break;
						case 0x57: { addKeyEvent( keyboard, keyCode_w, isDown, keyboard->shiftIsDown ); } break;
						case 0x58: { addKeyEvent( keyboard, keyCode_x, isDown, keyboard->shiftIsDown ); } break;
						case 0x59: { addKeyEvent( keyboard, keyCode_y, isDown, keyboard->shiftIsDown ); } break;
						case 0x5A: { addKeyEvent( keyboard, keyCode_z, isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_1:      { addKeyEvent( keyboard, keyCode_semicolon,    isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_PLUS:   { addKeyEvent( keyboard, keyCode_equal,        isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_COMMA:  { addKeyEvent( keyboard, keyCode_comma,        isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_MINUS:  { addKeyEvent( keyboard, keyCode_dash,         isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_PERIOD: { addKeyEvent( keyboard, keyCode_period,       isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_2:      { addKeyEvent( keyboard, keyCode_forwardSlash, isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_3:      { addKeyEvent( keyboard, keyCode_tilde,        isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_4:      { addKeyEvent( keyboard, keyCode_openBracket,  isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_5:      { addKeyEvent( keyboard, keyCode_backslash,    isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_6:      { addKeyEvent( keyboard, keyCode_closeBracket, isDown, keyboard->shiftIsDown ); } break;
						case VK_OEM_7:      { addKeyEvent( keyboard, keyCode_quote,        isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD0: { addKeyEvent( keyboard, keyCode_num0, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD1: { addKeyEvent( keyboard, keyCode_num1, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD2: { addKeyEvent( keyboard, keyCode_num2, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD3: { addKeyEvent( keyboard, keyCode_num3, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD4: { addKeyEvent( keyboard, keyCode_num4, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD5: { addKeyEvent( keyboard, keyCode_num5, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD6: { addKeyEvent( keyboard, keyCode_num6, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD7: { addKeyEvent( keyboard, keyCode_num7, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD8: { addKeyEvent( keyboard, keyCode_num8, isDown, keyboard->shiftIsDown ); } break;
						case VK_NUMPAD9: { addKeyEvent( keyboard, keyCode_num9, isDown, keyboard->shiftIsDown ); } break;
						default: {
							char string[ 128 ] = {};
							sprintf( string, "Key message received, but not processed: %lu %s\n", message.wParam, ( isDown ? "PRESSED" : "RELEASED" ) );
							OutputDebugString( string );
						} break;
					};
				}
			} break;
			
			default: {
				TranslateMessage( &message );
				DispatchMessage( &message );
			} break;
		}
	}
}

int32 CALLBACK
WinMain( HINSTANCE instance, HINSTANCE prevInstance, LPSTR commandLine, int32 windowShowCode ) {
	srand( ( uint32 )time( 0 ) );
	
	boo32 sleepFreqIsHiRes = ( timeBeginPeriod( 1 ) == TIMERR_NOERROR );
	
	LARGE_INTEGER perfFrequency;
	QueryPerformanceFrequency( &perfFrequency );
	
	WNDCLASS windowClass = {};
	windowClass.lpfnWndProc = win32_WindowProc;
	windowClass.hInstance = instance;
	windowClass.lpszClassName = "WindowClass";
	
	if( RegisterClass( &windowClass ) ) {
		int32 screen_width  = GetSystemMetrics( SM_CXSCREEN );
		int32 screen_height = GetSystemMetrics( SM_CYSCREEN );
		
		// HWND window = CreateWindowEx( 0, "WindowClass", "Caption", WS_VISIBLE, 0, 0, screen_width, screen_height, 0, 0, instance, 0 ); // WINDOWED
		HWND window = CreateWindowEx( 0, "WindowClass", "Caption", WS_EX_TOPMOST | WS_VISIBLE | WS_POPUP, 0, 0, screen_width, screen_height, 0, 0, instance, 0 ); // FULLSCREEN
		ShowCursor( FALSE );

		if( window ) {
			RECT client_rect = {};
			GetClientRect( window, &client_rect );
			
			int32 client_width  = client_rect.right  - client_rect.left;
			int32 client_height = client_rect.bottom - client_rect.top;
			
			vec2 client_dim = Vec2( ( flo32 )client_width, ( flo32 )client_height );
			vec2 client_halfDim = client_dim * 0.5f;
			rect client_bound = Rect( ( flo32 )client_rect.left, ( flo32 )client_rect.top, ( flo32 )client_rect.right, ( flo32 )client_rect.bottom );
			
			MEMORY platformMemory = {};
			platformMemory.size = ( sizeof( APP_STATE ) + sizeof( PLATFORM ) + APP_permMemorySize + APP_tempMemorySize );
			platformMemory.base = VirtualAlloc( 0, platformMemory.size, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE );
			
			APP_STATE * appState = _pushType( &platformMemory, APP_STATE );
			appState->isRunning = true;
			
			PLATFORM * win32_platform = _pushType( &platformMemory, PLATFORM );
			win32_platform->permMemory = { _pushSize( &platformMemory, APP_permMemorySize ), APP_permMemorySize };
			win32_platform->tempMemory = { _pushSize( &platformMemory, APP_tempMemorySize ), APP_tempMemorySize };
			win32_platform->readFile   = win32_readFile;
			win32_platform->writeFile  = win32_writeFile;
			win32_platform->targetSec  = 1.0f / APP_targetFPS;
			win32_platform->backbuffer.width  = APP_width;
			win32_platform->backbuffer.height = APP_height;
			
			MEMORY * permMemory = &win32_platform->permMemory;
			MEMORY * tempMemory = &win32_platform->tempMemory;
			
			HDC win32_deviceContext = GetDC( window );
			
			BITMAPINFO bitmap_info = {};
			bitmap_info.bmiHeader.biSize        = sizeof( BITMAPINFOHEADER );
			bitmap_info.bmiHeader.biWidth       = client_width;
			bitmap_info.bmiHeader.biHeight      = client_height;
			bitmap_info.bmiHeader.biPlanes      = 1;
			bitmap_info.bmiHeader.biBitCount    = 32;
			bitmap_info.bmiHeader.biCompression = BI_RGB;
			
			vec2 app_dim     = Vec2( ( flo32 )APP_width, ( flo32 )APP_height );
			vec2 app_halfDim = app_dim * 0.5f;
			rect app_bound   = rectBLD( Vec2( 0.0f, 0.0f ), app_dim );
			appState->app_dim     = app_dim;
			appState->app_halfDim = app_halfDim;
			appState->app_bound   = app_bound;
			appState->targetSec   = win32_platform->targetSec;
			
			flo32 targetMSElapsed = win32_platform->targetSec * 1000.0f;
			
			BACKBUFFER * backbuffer = &win32_platform->backbuffer;
			
			int32 screen_bufferSize = client_width * client_height * sizeof( uint32 );
			uint32 * screen_backbuffer = ( uint32 * )VirtualAlloc( 0, screen_bufferSize, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE );
			{
				uint32 * ptr = screen_backbuffer;
				for( int32 iter = 0; iter < ( client_width * client_height ); iter++ ) {
					*ptr++ = 0xFF000000;
				}
			}
			
			while( appState->isRunning ) {
				int64  start_counter = win32_getPerfCount();
				// uint64 start_clock   = __rdtsc();
				
				win32_processPendingMessages( win32_platform, window );
				
				// if( wasPressed( &win32_platform->keyboard, keyCode_p ) ) {
					// appState->isRunning = false;
				// }
				
				APP_updateAndRender( appState, win32_platform );
				
				flo32 game_width  = ( flo32 )APP_width;
				flo32 game_height = ( flo32 )APP_height;
				flo32 game_aspectRatio = game_width / game_height;
				
				flo32 screen_width  = ( flo32 )client_width;
				flo32 screen_height = ( flo32 )client_height;
				flo32 screen_aspectRatio = screen_width / screen_height;
				
				boo32 adjustHorizontal = ( game_aspectRatio < screen_aspectRatio );
				flo32 ratio = adjustHorizontal ? ( screen_height / game_height ) : ( screen_width / game_width );
				
				int32 bound_x = 0;
				int32 bound_y = 0;
				int32 bound_width  = ( int32 )( ceilf( game_width  * ratio ) );
				int32 bound_height = ( int32 )( ceilf( game_height * ratio ) );
				
				bound_width  = ( bound_width  / 4 ) * 4;
				bound_height = ( bound_height / 4 ) * 4;
				
				if( adjustHorizontal ) {
					bound_x = ( int32 )( ( client_width  - bound_width  ) / 2 );
				} else {
					bound_y = ( int32 )( ( client_height - bound_height ) / 2 );
				}
				
				{
					uint32 pitch = client_width;
					uint32 backbuffer_width  = backbuffer->width;
					uint32 backbuffer_height = backbuffer->height;
					
					uint32 * src     = backbuffer->memory;
					uint32 * destRow = screen_backbuffer + bound_y * pitch + bound_x;
					
					__m128  _one  = _mm_set1_ps( 1.0f );
					__m128i _onei = _mm_set1_epi32( 1 );
					__m128i _mask = _mm_set1_epi32( 0x000000FF );
					__m128  _invBoundWidth   = _mm_set1_ps( 1.0f / ( flo32 )bound_width );
					__m128  _backbufferWidth = _mm_set1_ps( ( flo32 )backbuffer->width  );
					__m128i _blendC_a = _mm_set1_epi32( 0xFF000000 );
					
					for( int32 y = 0; y < bound_height; y++ ) {
						uint32 * pixel = destRow;
						
						flo32 yf = ( ( flo32 )y / ( flo32 )bound_height * ( flo32 )backbuffer_height );
						int32 yi = ( int32 )yf;
						flo32 tY = ( yf - ( flo32 )yi );
						
						__m128 _tY    = _mm_set1_ps( tY        );
						__m128 _invtY = _mm_set1_ps( 1.0f - tY );
						
						int32 ya = yi;
						int32 yb = yi + 1;
						
						int32 pitchA = ya * backbuffer_width;
						int32 pitchB = yb * backbuffer_width;
						
						__m128i _pitchA = _mm_set1_epi32( pitchA );
						__m128i _pitchB = _mm_set1_epi32( pitchB );
						
						for( int32 x = 0; x < bound_width; x += 4 ) {							
							__m128i _pixelXi = _mm_set_epi32( x + 3, x + 2, x + 1, x );
							__m128  _pixelXf = _mm_cvtepi32_ps( _pixelXi );
							
							__m128  _texelXf = _mm_mul_ps( _mm_mul_ps( _pixelXf, _invBoundWidth ), _backbufferWidth );
							__m128i _texelXi = _mm_cvttps_epi32( _texelXf );
							
							__m128 _tX    = _mm_sub_ps( _texelXf, _mm_cvtepi32_ps( _texelXi ) );
							__m128 _invtX = _mm_sub_ps( _one, _tX );
							
							__m128i _texelXa =                _texelXi;
							__m128i _texelXb = _mm_add_epi32( _texelXi, _onei );
							
							__m128i _fetchA = _mm_add_epi32( _pitchA, _texelXa );
							__m128i _fetchB = _mm_add_epi32( _pitchA, _texelXb );
							__m128i _fetchC = _mm_add_epi32( _pitchB, _texelXa );
							__m128i _fetchD = _mm_add_epi32( _pitchB, _texelXb );
							
						#define _mmGetI( vector, index ) ( ( int32 * )&vector )[ index ]
							__m128i _texelA = _mm_set_epi32( *( src + _mmGetI( _fetchA, 3 ) ), *( src + _mmGetI( _fetchA, 2 ) ), *( src + _mmGetI( _fetchA, 1 ) ), *( src + _mmGetI( _fetchA, 0 ) ) );
							__m128i _texelB = _mm_set_epi32( *( src + _mmGetI( _fetchB, 3 ) ), *( src + _mmGetI( _fetchB, 2 ) ), *( src + _mmGetI( _fetchB, 1 ) ), *( src + _mmGetI( _fetchB, 0 ) ) );
							__m128i _texelC = _mm_set_epi32( *( src + _mmGetI( _fetchC, 3 ) ), *( src + _mmGetI( _fetchC, 2 ) ), *( src + _mmGetI( _fetchC, 1 ) ), *( src + _mmGetI( _fetchC, 0 ) ) );
							__m128i _texelD = _mm_set_epi32( *( src + _mmGetI( _fetchD, 3 ) ), *( src + _mmGetI( _fetchD, 2 ) ), *( src + _mmGetI( _fetchD, 1 ) ), *( src + _mmGetI( _fetchD, 0 ) ) );
														
							__m128 _texelA_r = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _texelA, 16 ), _mask ) );
							__m128 _texelA_g = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _texelA,  8 ), _mask ) );
							__m128 _texelA_b = _mm_cvtepi32_ps( _mm_and_si128(                 _texelA,       _mask ) );
							
							__m128 _texelB_r = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _texelB, 16 ), _mask ) );
							__m128 _texelB_g = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _texelB,  8 ), _mask ) );
							__m128 _texelB_b = _mm_cvtepi32_ps( _mm_and_si128(                 _texelB,       _mask ) );
							
							__m128 _texelC_r = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _texelC, 16 ), _mask ) );
							__m128 _texelC_g = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _texelC,  8 ), _mask ) );
							__m128 _texelC_b = _mm_cvtepi32_ps( _mm_and_si128(                 _texelC,       _mask ) );
							
							__m128 _texelD_r = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _texelD, 16 ), _mask ) );
							__m128 _texelD_g = _mm_cvtepi32_ps( _mm_and_si128( _mm_srli_epi32( _texelD,  8 ), _mask ) );
							__m128 _texelD_b = _mm_cvtepi32_ps( _mm_and_si128(                 _texelD,       _mask ) );
							
							__m128 _blendA_r = _mm_add_ps( _mm_mul_ps( _invtX, _texelA_r ), _mm_mul_ps( _tX, _texelB_r ) );
							__m128 _blendA_g = _mm_add_ps( _mm_mul_ps( _invtX, _texelA_g ), _mm_mul_ps( _tX, _texelB_g ) );
							__m128 _blendA_b = _mm_add_ps( _mm_mul_ps( _invtX, _texelA_b ), _mm_mul_ps( _tX, _texelB_b ) );
							
							__m128 _blendB_r = _mm_add_ps( _mm_mul_ps( _invtX, _texelC_r ), _mm_mul_ps( _tX, _texelD_r ) );
							__m128 _blendB_g = _mm_add_ps( _mm_mul_ps( _invtX, _texelC_g ), _mm_mul_ps( _tX, _texelD_g ) );
							__m128 _blendB_b = _mm_add_ps( _mm_mul_ps( _invtX, _texelC_b ), _mm_mul_ps( _tX, _texelD_b ) );
							
							__m128 _blendC_r = _mm_add_ps( _mm_mul_ps( _invtY, _blendA_r ), _mm_mul_ps( _tY, _blendB_r ) );
							__m128 _blendC_g = _mm_add_ps( _mm_mul_ps( _invtY, _blendA_g ), _mm_mul_ps( _tY, _blendB_g ) );
							__m128 _blendC_b = _mm_add_ps( _mm_mul_ps( _invtY, _blendA_b ), _mm_mul_ps( _tY, _blendB_b ) );
							
							__m128i _write_color =                                             _mm_cvttps_epi32( _blendC_b );
									  _write_color = _mm_or_si128( _write_color, _mm_slli_epi32( _mm_cvttps_epi32( _blendC_g ),  8 ) );
									  _write_color = _mm_or_si128( _write_color, _mm_slli_epi32( _mm_cvttps_epi32( _blendC_r ), 16 ) );
									  _write_color = _mm_or_si128( _write_color,                                  _blendC_a         );
							
							_mm_storeu_si128( ( __m128i * )pixel, _write_color );
							
							pixel += 4;
						}
						destRow += pitch;
					}
				}
				StretchDIBits( win32_deviceContext, 0, 0, client_width, client_height, 0, 0, client_width, client_height, screen_backbuffer, &bitmap_info, DIB_RGB_COLORS, SRCCOPY );
				
				int64  end_counter = win32_getPerfCount();
				// uint64 end_clock   = __rdtsc();
				
				flo32   msElapsed = win32_getMSElapsed( start_counter, end_counter, perfFrequency );
				// uint64 cycElapsed = end_clock - start_clock;
				
				// char profile_string[ 128 ] = {};
				// sprintf( profile_string, "ms:%8.04f,  cyc: %10I64u\n", msElapsed, cycElapsed );
				// OutputDebugString( profile_string );
				
				if( msElapsed < targetMSElapsed ) {
					if( sleepFreqIsHiRes ) {
						DWORD msToSleep = ( DWORD )( targetMSElapsed - msElapsed );
						if( msToSleep > 0 ) {
							Sleep( msToSleep );
						}
					}
					while( msElapsed < targetMSElapsed ) {
						msElapsed = win32_getMSElapsed( start_counter, win32_getPerfCount(), perfFrequency );
					}
				}
				
			}
		}
	}
   
	return 0;
}